# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words

TEST_CASES_TO_CURRENCY_EUR = (
    (1.00, 'ein Euro und null Cent'),
    (2.01, 'zwei Euro und ein Cent'),
    (8.10, 'acht Euro und zehn Cent'),
    (12.26, 'zwölf Euro und sechsundzwanzig Cent'),
    (21.29, 'einundzwanzig Euro und neunundzwanzig Cent'),
    (81.25, 'einundachtzig Euro und fünfundzwanzig Cent'),
    (100.00, 'einhundert Euro und null Cent'),
)

TEST_CASES_TO_CURRENCY_USD = (
    (1.00, 'ein Dollar und null Cent'),
    (2.01, 'zwei Dollar und ein Cent'),
    (8.10, 'acht Dollar und zehn Cent'),
    (12.26, 'zwölf Dollar und sechsundzwanzig Cent'),
    (21.29, 'einundzwanzig Dollar und neunundzwanzig Cent'),
    (81.25, 'einundachtzig Dollar und fünfundzwanzig Cent'),
    (100.00, 'einhundert Dollar und null Cent'),
)

TEST_CASES_TO_CURRENCY_GBP = (
    (1.00, 'ein Pfund und null Pence'),
    (2.01, 'zwei Pfund und ein Penny'),
    (8.10, 'acht Pfund und zehn Pence'),
    (12.26, 'zwölf Pfund und sechsundzwanzig Pence'),
    (21.29, 'einundzwanzig Pfund und neunundzwanzig Pence'),
    (81.25, 'einundachtzig Pfund und fünfundzwanzig Pence'),
    (100.00, 'einhundert Pfund und null Pence'),
)

TEST_CASES_TO_CURRENCY_DEM = (
    (1.00, 'ein Mark und null Pfennig'),
    (2.01, 'zwei Mark und ein Pfennig'),
    (8.10, 'acht Mark und zehn Pfennig'),
    (12.26, 'zwölf Mark und sechsundzwanzig Pfennig'),
    (21.29, 'einundzwanzig Mark und neunundzwanzig Pfennig'),
    (81.25, 'einundachtzig Mark und fünfundzwanzig Pfennig'),
    (100.00, 'einhundert Mark und null Pfennig'),
)


class Num2WordsDETest(TestCase):

    def test_ordinal_less_than_twenty(self):
        self.assertEqual(num2words(0, ordinal=True, lang='de'), "nullte")
        self.assertEqual(num2words(1, ordinal=True, lang='de'), "erste")
        self.assertEqual(num2words(7, ordinal=True, lang='de'), "siebte")
        self.assertEqual(num2words(8, ordinal=True, lang='de'), "achte")
        self.assertEqual(num2words(12, ordinal=True, lang='de'), "zwölfte")
        self.assertEqual(num2words(17, ordinal=True, lang='de'), "siebzehnte")

    def test_ordinal_more_than_twenty(self):
        self.assertEqual(
            num2words(81, ordinal=True, lang='de'), "einundachtzigste"
        )

    def test_ordinal_at_crucial_number(self):
        self.assertEqual(
            num2words(100, ordinal=True, lang='de'), "hundertste"
        )
        self.assertEqual(
            num2words(1000, ordinal=True, lang='de'), "tausendste"
        )
        self.assertEqual(
            num2words(4000, ordinal=True, lang='de'), "viertausendste"
        )
        self.assertEqual(
            num2words(1000000, ordinal=True, lang='de'), "millionste"
        )
        self.assertEqual(
            num2words(2000000, ordinal=True, lang='de'), "zweimillionste"
        )
        self.assertEqual(
            num2words(1000000000, ordinal=True, lang='de'), "milliardste"
        )
        self.assertEqual(
            num2words(5000000000, ordinal=True, lang='de'),
            "fünfmilliardste"
        )

    def test_cardinal_at_some_numbers(self):
        self.assertEqual(num2words(100, lang='de'), "einhundert")
        self.assertEqual(num2words(1000, lang='de'), "eintausend")
        self.assertEqual(num2words(5000, lang='de'), "fünftausend")
        self.assertEqual(num2words(10000, lang='de'), "zehntausend")
        self.assertEqual(num2words(1000000, lang='de'), "eine Million")
        self.assertEqual(num2words(2000000, lang='de'), "zwei Millionen")
        self.assertEqual(num2words(4000000000, lang='de'), "vier Milliarden")
        self.assertEqual(num2words(1000000000, lang='de'), "eine Milliarde")

    def test_cardinal_for_decimal_number(self):
        self.assertEqual(
            num2words(3.486, lang='de'), "drei Komma vier acht sechs"
        )

    def test_giant_cardinal_for_merge(self):
        self.assertEqual(
            num2words(4500072900000111, lang='de'),
            "vier Billiarden fünfhundert Billionen " +
            "zweiundsiebzig Milliarden neunhundert Millionen einhundertelf"
        )

    def test_ordinal_num(self):
        self.assertEqual(num2words(7, to="ordinal_num", lang='de'), "7.")
        self.assertEqual(num2words(81, to="ordinal_num", lang='de'), "81.")

    def test_ordinal_for_negative_numbers(self):
        self.assertRaises(TypeError, num2words, -12, ordinal=True, lang='de')

    def test_ordinal_for_floating_numbers(self):
        self.assertRaises(TypeError, num2words, 2.453, ordinal=True, lang='de')

    def test_currency_eur(self):
        for test in TEST_CASES_TO_CURRENCY_EUR:
            self.assertEqual(
                num2words(test[0], lang='de', to='currency', currency='EUR'),
                test[1]
            )

    def test_currency_usd(self):
        for test in TEST_CASES_TO_CURRENCY_USD:
            self.assertEqual(
                num2words(test[0], lang='de', to='currency', currency='USD'),
                test[1]
            )

    def test_currency_dem(self):
        for test in TEST_CASES_TO_CURRENCY_DEM:
            self.assertEqual(
                num2words(test[0], lang='de', to='currency', currency='DEM'),
                test[1]
            )

    def test_currency_gbp(self):
        for test in TEST_CASES_TO_CURRENCY_GBP:
            self.assertEqual(
                num2words(test[0], lang='de', to='currency', currency='GBP'),
                test[1]
            )

    def test_year(self):
        self.assertEqual(num2words(2002, to='year', lang='de'),
                         'zweitausendzwei')

    def test_year_before_2000(self):
        self.assertEqual(num2words(1780, to='year', lang='de'),
                         'siebzehnhundertachtzig')



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from unittest import TestCase

from num2words import num2words


class Num2WordsAMTest(TestCase):
    def test_cardinal(self):
        self.assertEqual(num2words(100, lang='am'), 'መቶ')
        self.assertEqual(num2words(100000, lang='am'), 'አንድ መቶ ሺህ')
        self.assertEqual(num2words(101, lang='am'), 'አንድ መቶ አንድ')

    def test_and_join_199(self):
        self.assertEqual(num2words(199, lang='am'), 'አንድ መቶ ዘጠና ዘጠኝ')

    def test_to_ordinal(self):
        self.assertEqual(
            num2words(1, lang='am', to='ordinal'),
            'አንደኛ'
        )
        self.assertEqual(
            num2words(13, lang='am', to='ordinal'),
            'አሥራ ሦስተኛ'
        )
        self.assertEqual(
            num2words(22, lang='am', to='ordinal'),
            'ሃያ ሁለተኛ'
        )
        self.assertEqual(
            num2words(10000, lang='am', to='ordinal'),
            'አሥር ሺህኛ'
        )

    def test_to_ordinal_num(self):
        self.assertEqual(num2words(10, lang='am', to='ordinal_num'), '10ኛ')
        self.assertEqual(num2words(21, lang='am', to='ordinal_num'), '21ኛ')
        self.assertEqual(num2words(102, lang='am', to='ordinal_num'), '102ኛ')

    def test_cardinal_for_float_number(self):
        self.assertEqual(num2words(12.5, lang='am'), 'አሥራ ሁለት ነጥብ አምስት')
        self.assertEqual(num2words(12.51, lang='am'), 'አሥራ ሁለት ነጥብ አምስት አንድ')
        self.assertEqual(num2words(12.53, lang='am'), 'አሥራ ሁለት ነጥብ አምስት ሦስት')

    def test_to_overflow(self):
        with self.assertRaises(OverflowError):
            num2words('1000000000000000000000000000000000000000000000000000000'
                      '0000000000000000000000000000000000000000000000000000000'
                      '0000000000000000000000000000000000000000000000000000000'
                      '0000000000000000000000000000000000000000000000000000000'
                      '0000000000000000000000000000000000000000000000000000000'
                      '00000000000000000000000000000000', lang='am')

    def test_to_currency(self):
        self.assertEqual(
            num2words('38.4', lang='am', to='currency', cents=False,
                      currency='ETB'), 'ሠላሳ ስምንት ብር ከ 40 ሳንቲም'
        )
        self.assertEqual(
            num2words('0', lang='am', to='currency', separator=' እና',
                      cents=True, currency='ETB'), 'ዜሮ ብር እና ዜሮ ሳንቲም'
        )

        self.assertEqual(
            num2words('1.50', lang='am', to='currency', cents=True,
                      currency='ETB'), 'አንድ ብር ከ አምሳ ሳንቲም'
        )

    def test_to_year(self):
        self.assertEqual(num2words(1990, lang='am', to='year'),
                         'አሥራ ዘጠኝ መቶ ዘጠና')
        self.assertEqual(num2words(5555, lang='am', to='year'),
                         'አምሳ አምስት መቶ አምሳ አምስት')
        self.assertEqual(num2words(2017, lang='am', to='year'),
                         'ሁለት ሺህ አሥራ ሰባት')
        self.assertEqual(num2words(1066, lang='am', to='year'),
                         'አንድ ሺህ ስድሳ ስድስት')
        self.assertEqual(num2words(1865, lang='am', to='year'),
                         'አሥራ ስምንት መቶ ስድሳ አምስት')




# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsSKTest(TestCase):
    def test_cardinal(self):
        self.assertEqual(num2words(100, lang='sk'), "sto")
        self.assertEqual(num2words(101, lang='sk'), "stojeden")
        self.assertEqual(num2words(110, lang='sk'), "stodesať")
        self.assertEqual(num2words(115, lang='sk'), "stopätnásť")
        self.assertEqual(num2words(123, lang='sk'), "stodvadsaťtri")
        self.assertEqual(num2words(1000, lang='sk'), "tisíc")
        self.assertEqual(num2words(1001, lang='sk'), "tisícjeden")
        self.assertEqual(num2words(2012, lang='sk'), "dvetisícdvanásť")
        self.assertEqual(
            num2words(10.02, lang='sk'),
            "desať celých nula dva"
        )
        self.assertEqual(
            num2words(15.007, lang='sk'),
            "pätnásť celých nula nula sedem"
        )
        self.assertEqual(
            num2words(12519.85, lang='sk'),
            "dvanásťtisícpäťstodevätnásť celých osemdesiatpäť"
        )
        self.assertEqual(
            num2words(123.50, lang='sk'),
            "stodvadsaťtri celých päť"
        )
        self.assertEqual(
            num2words(1234567890, lang='sk'),
            "miliarda dvestotridsaťštyri miliónov päťstošesťdesiat"
            "sedemtisícosemstodeväťdesiat"
        )
        self.assertEqual(
            num2words(215461407892039002157189883901676, lang='sk'),
            "dvestopätnásť kvintiliónov štyristošesťdesiatjeden kvadriliárd "
            "štyristosedem kvadriliónov osemstodeväťdesiatdva triliárd "
            "tridsaťdeväť triliónov dve biliardy stopäťdesiatsedem biliónov "
            "stoosemdesiatdeväť miliárd osemstoosemdesiattri miliónov "
            "deväťstojedentisícšesťstosedemdesiatšesť"
        )
        self.assertEqual(
            num2words(719094234693663034822824384220291, lang='sk'),
            "sedemstodevätnásť kvintiliónov deväťdesiatštyri kvadriliárd "
            "dvestotridsaťštyri kvadriliónov šesťstodeväťdesiattri triliárd "
            "šesťstošesťdesiattri triliónov tridsaťštyri biliárd "
            "osemstodvadsaťdva biliónov osemstodvadsaťštyri miliárd "
            "tristoosemdesiatštyri miliónov "
            "dvestodvadsaťtisícdvestodeväťdesiatjeden"
        )

    def test_to_ordinal(self):
        # @TODO: implement to_ordinal
        with self.assertRaises(NotImplementedError):
            num2words(1, lang='sk', to='ordinal')

    def test_currency(self):
        self.assertEqual(
            num2words(10.0, lang='sk', to='currency', currency='EUR'),
            "desať eur, nula centov")
        self.assertEqual(
            num2words(1234.56, lang='sk', to='currency', currency='EUR'),
            "tisícdvestotridsaťštyri eur, päťdesiatšesť centov")
        self.assertEqual(
            num2words(101.11, lang='sk', to='currency', currency='EUR',
                      separator=' a'),
            "stojeden eur a jedenásť centov")
        self.assertEqual(
            num2words(-12519.85, lang='sk', to='currency', cents=False),
            "mínus dvanásťtisícpäťstodevätnásť eur, 85 centov"
        )
        self.assertEqual(
            num2words(19.50, lang='sk', to='currency', cents=False),
            "devätnásť eur, 50 centov"
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsRUTest(TestCase):

    def test_cardinal(self):
        self.assertEqual(num2words(100, lang='ru'), "сто")
        self.assertEqual(num2words(101, lang='ru'), "сто один")
        self.assertEqual(num2words(110, lang='ru'), "сто десять")
        self.assertEqual(num2words(115, lang='ru'), "сто пятнадцать")
        self.assertEqual(num2words(123, lang='ru'), "сто двадцать три")
        self.assertEqual(num2words(1000, lang='ru'), "одна тысяча")
        self.assertEqual(num2words(1001, lang='ru'), "одна тысяча один")
        self.assertEqual(num2words(2012, lang='ru'), "две тысячи двенадцать")
        self.assertEqual(
            num2words(12519.85, lang='ru'),
            "двенадцать тысяч пятьсот девятнадцать целых восемьдесят пять "
            "сотых"
        )
        self.assertEqual(
            num2words(1234567890, lang='ru'),
            "один миллиард двести тридцать четыре миллиона пятьсот "
            "шестьдесят семь тысяч восемьсот девяносто")
        self.assertEqual(
            num2words(215461407892039002157189883901676, lang='ru'),
            "двести пятнадцать нониллионов четыреста шестьдесят один "
            "октиллион четыреста семь септиллионов восемьсот девяносто "
            "два секстиллиона тридцать девять квинтиллионов два квадриллиона "
            "сто пятьдесят семь триллионов сто восемьдесят девять миллиардов "
            "восемьсот восемьдесят три миллиона девятьсот одна тысяча "
            "шестьсот семьдесят шесть")
        self.assertEqual(
            num2words(719094234693663034822824384220291, lang='ru'),
            "семьсот девятнадцать нониллионов девяносто четыре октиллиона "
            "двести тридцать четыре септиллиона шестьсот девяносто три "
            "секстиллиона шестьсот шестьдесят три квинтиллиона тридцать "
            "четыре квадриллиона восемьсот двадцать два триллиона восемьсот "
            "двадцать четыре миллиарда триста восемьдесят четыре миллиона "
            "двести двадцать тысяч двести девяносто один")
        self.assertEqual(num2words(5, lang='ru'), "пять")
        self.assertEqual(num2words(15, lang='ru'), "пятнадцать")
        self.assertEqual(num2words(154, lang='ru'), "сто пятьдесят четыре")
        self.assertEqual(
            num2words(1135, lang='ru'), "одна тысяча сто тридцать пять"
        )
        self.assertEqual(
            num2words(418531, lang='ru'),
            "четыреста восемнадцать тысяч пятьсот тридцать один"
        )
        self.assertEqual(
            num2words(1000139, lang='ru'), "один миллион сто тридцать девять"
        )
        self.assertEqual(num2words(-1, lang='ru'), "минус один")
        self.assertEqual(num2words(-15, lang='ru'), "минус пятнадцать")
        self.assertEqual(num2words(-100, lang='ru'), "минус сто")

    def test_cardinal_feminine(self):
        self.assertEqual(num2words(1, lang='ru', gender='f'), 'одна')
        self.assertEqual(num2words(2, lang='ru', gender='f'), 'две')
        self.assertEqual(num2words(3, lang='ru', gender='f'), 'три')
        self.assertEqual(num2words(100, lang='ru', gender='f'), "сто")
        self.assertEqual(num2words(101, lang='ru', gender='f'), "сто одна")
        self.assertEqual(num2words(110, lang='ru', gender='f'), "сто десять")
        self.assertEqual(
            num2words(115, lang='ru', gender='f'), "сто пятнадцать"
        )
        self.assertEqual(
            num2words(122, lang='ru', gender='f'), "сто двадцать две"
        )
        self.assertEqual(
            num2words(125.1, lang='ru', gender='f'),
            'сто двадцать пять целых одна десятая'
        )
        self.assertEqual(num2words(-1, lang='ru', gender='f'), "минус одна")
        self.assertEqual(num2words(-100, lang='ru', gender='f'), "минус сто")

    def test_cardinal_neuter(self):
        self.assertEqual(num2words(1, lang='ru', gender='n'), 'одно')
        self.assertEqual(num2words(2, lang='ru', gender='n'), 'два')
        self.assertEqual(num2words(3, lang='ru', gender='n'), 'три')
        self.assertEqual(num2words(100, lang='ru', gender='n'), "сто")
        self.assertEqual(num2words(101, lang='ru', gender='n'), "сто одно")
        self.assertEqual(num2words(110, lang='ru', gender='n'), "сто десять")
        self.assertEqual(
            num2words(115, lang='ru', gender='n'), "сто пятнадцать"
        )
        self.assertEqual(
            num2words(122, lang='ru', gender='n'), "сто двадцать два"
        )
        self.assertEqual(
            num2words(125.1, lang='ru', gender='n'),
            'сто двадцать пять целых одна десятая')
        self.assertEqual(num2words(-1, lang='ru', gender='n'), "минус одно")
        self.assertEqual(num2words(-100, lang='ru', gender='n'), "минус сто")

    def test_floating_point(self):
        self.assertEqual(num2words(5.2, lang='ru'), "пять целых две десятых")
        self.assertEqual(num2words(5.0, lang='ru'), "пять целых ноль десятых")
        self.assertEqual(num2words(5.10, lang='ru'), "пять целых одна десятая")
        self.assertEqual(num2words("5.10", lang='ru'),
                         "пять целых десять сотых")
        self.assertEqual(num2words(1.001, lang='ru'),
                         "одна целая одна тысячная")
        self.assertEqual(num2words(1.011, lang='ru'),
                         "одна целая одиннадцать тысячных")
        self.assertEqual(num2words(10.02, lang='ru'),
                         "десять целых две сотых")
        self.assertEqual(num2words(15.007, lang='ru'),
                         "пятнадцать целых семь тысячных")
        self.assertEqual(num2words(561.42, lang='ru'),
                         "пятьсот шестьдесят одна целая сорок две сотых")
        self.assertEqual(num2words(561.00001, lang='ru'),
                         "пятьсот шестьдесят одна целая одна стотысячная")

    def test_to_ordinal(self):
        self.assertEqual(
            num2words(0, lang='ru', to='ordinal'),
            'нулевой'
        )
        self.assertEqual(
            num2words(1, lang='ru', to='ordinal'),
            'первый'
        )
        self.assertEqual(
            num2words(5, lang='ru', to='ordinal'),
            'пятый'
        )
        self.assertEqual(
            num2words(10, lang='ru', to='ordinal'),
            'десятый'
        )

        self.assertEqual(
            num2words(13, lang='ru', to='ordinal'),
            'тринадцатый'
        )
        self.assertEqual(
            num2words(20, lang='ru', to='ordinal'),
            'двадцатый'
        )
        self.assertEqual(
            num2words(23, lang='ru', to='ordinal'),
            'двадцать третий'
        )
        self.assertEqual(
            num2words(40, lang='ru', to='ordinal'),
            'сороковой'
        )
        self.assertEqual(
            num2words(70, lang='ru', to='ordinal'),
            'семидесятый'
        )
        self.assertEqual(
            num2words(100, lang='ru', to='ordinal'),
            'сотый'
        )
        self.assertEqual(
            num2words(136, lang='ru', to='ordinal'),
            'сто тридцать шестой'
        )
        self.assertEqual(
            num2words(500, lang='ru', to='ordinal'),
            'пятисотый'
        )
        self.assertEqual(
            num2words(1000, lang='ru', to='ordinal'),
            'тысячный'
        )
        self.assertEqual(
            num2words(1001, lang='ru', to='ordinal'),
            'тысяча первый'
        )
        self.assertEqual(
            num2words(1060, lang='ru', to='ordinal'),
            'тысяча шестидесятый'
        )
        self.assertEqual(
            num2words(2000, lang='ru', to='ordinal'),
            'двухтысячный'
        )
        self.assertEqual(
            num2words(10000, lang='ru', to='ordinal'),
            'десятитысячный'
        )
        self.assertEqual(
            num2words(90000, lang='ru', to='ordinal'),
            'девяностотысячный'
        )
        self.assertEqual(
            num2words(21000, lang='ru', to='ordinal'),
            'двадцатиоднотысячный'
        )
        self.assertEqual(
            num2words(130000, lang='ru', to='ordinal'),
            'стотридцатитысячный'
        )
        self.assertEqual(
            num2words(135000, lang='ru', to='ordinal'),
            'стотридцатипятитысячный'
        )
        self.assertEqual(
            num2words(135100, lang='ru', to='ordinal'),
            'сто тридцать пять тысяч сотый'
        )
        self.assertEqual(
            num2words(135120, lang='ru', to='ordinal'),
            'сто тридцать пять тысяч сто двадцатый'
        )
        self.assertEqual(
            num2words(135121, lang='ru', to='ordinal'),
            'сто тридцать пять тысяч сто двадцать первый'
        )
        self.assertEqual(
            num2words(190000, lang='ru', to='ordinal'),
            'стодевяностотысячный'
        )
        self.assertEqual(
            num2words(1000000, lang='ru', to='ordinal'),
            'миллионный'
        )
        self.assertEqual(
            num2words(2000000, lang='ru', to='ordinal'),
            'двухмиллионный'
        )
        self.assertEqual(
            num2words(5135000, lang='ru', to='ordinal'),
            'пять миллионов стотридцатипятитысячный'
        )
        self.assertEqual(
            num2words(21000000, lang='ru', to='ordinal'),
            'двадцатиодномиллионный'
        )
        self.assertEqual(
            num2words(1000000000, lang='ru', to='ordinal'),
            'миллиардный'
        )
        self.assertEqual(
            num2words(123456000000, lang='ru', to='ordinal'),
            'сто двадцать три миллиарда четырёхсотпятидесятишестимиллионный'
        )

    def test_to_ordinal_feminine(self):
        self.assertEqual(
            num2words(1, lang='ru', to='ordinal', gender='f'), 'первая'
        )
        self.assertEqual(
            num2words(3, lang='ru', to='ordinal', gender='f'), 'третья'
        )
        self.assertEqual(
            num2words(10, lang='ru', to='ordinal', gender='f'), 'десятая'
        )
        self.assertEqual(
            num2words(23, lang='ru', to='ordinal', gender='f'),
            'двадцать третья'
        )
        self.assertEqual(
            num2words(1000, lang='ru', to='ordinal', gender='f'), 'тысячная'
        )
        self.assertEqual(
            num2words(2000000, lang='ru', to='ordinal', gender='f'),
            'двухмиллионная'
        )

    def test_to_ordinal_neuter(self):
        self.assertEqual(
            num2words(1, lang='ru', to='ordinal', gender='n'), 'первое'
        )
        self.assertEqual(
            num2words(3, lang='ru', to='ordinal', gender='n'), 'третье'
        )
        self.assertEqual(
            num2words(10, lang='ru', to='ordinal', gender='n'), 'десятое'
        )
        self.assertEqual(
            num2words(23, lang='ru', to='ordinal', gender='n'),
            'двадцать третье'
        )
        self.assertEqual(
            num2words(1000, lang='ru', to='ordinal', gender='n'), 'тысячное'
        )
        self.assertEqual(
            num2words(2000000, lang='ru', to='ordinal', gender='n'),
            'двухмиллионное'
        )

    def test_cardinal_cases(self):
        self.assertEqual(
            num2words(1, lang='ru', case='nominative'), 'один')
        self.assertEqual(
            num2words(1, lang='ru', case='genitive'), 'одного')
        self.assertEqual(
            num2words(1, lang='ru', case='a', plural=True, animate=False),
            'одни')
        self.assertEqual(
            num2words(2, lang='ru', case='a', gender='f', animate=True),
            'двух')
        self.assertEqual(
            num2words(2, lang='ru', case='a', gender='f', animate=False),
            'две')
        self.assertEqual(
            num2words(100, lang='ru', case='g'),
            'ста')
        self.assertEqual(
            num2words(122, lang='ru', case='d'),
            'ста двадцати двум')
        self.assertEqual(
            num2words(1000, lang='ru', case='p'),
            'одной тысяче')
        self.assertEqual(
            num2words(1122, lang='ru', case='p'),
            'одной тысяче ста двадцати двух')
        self.assertEqual(
            num2words(1211, lang='ru', case='i', gender='f'),
            'одной тысячей двумястами одиннадцатью')
        self.assertEqual(
            num2words(5121000, lang='ru', case='i'),
            'пятью миллионами ста двадцатью одной тысячей')

    def test_ordinal_cases(self):
        self.assertEqual(
            num2words(1, lang='ru', to='ordinal', case='nominative'), 'первый')
        self.assertEqual(
            num2words(1, lang='ru', to='ordinal', case='genitive'), 'первого')
        self.assertEqual(
            num2words(1, lang='ru', to='ordinal', case='a', plural=True,
                      animate=False),
            'первые')
        self.assertEqual(
            num2words(2, lang='ru', to='ordinal', case='a', gender='f',
                      animate=True),
            'вторую')
        self.assertEqual(
            num2words(2, lang='ru', to='ordinal', case='a', gender='f',
                      animate=False),
            'вторую')
        self.assertEqual(
            num2words(100, lang='ru', to='ordinal', case='g'),
            'сотого')
        self.assertEqual(
            num2words(122, lang='ru', to='ordinal', case='d'),
            'сто двадцать второму')
        self.assertEqual(
            num2words(1000, lang='ru', to='ordinal', case='p'),
            'тысячном')
        self.assertEqual(
            num2words(1122, lang='ru', to='ordinal', case='p'),
            'тысяча сто двадцать втором')
        self.assertEqual(
            num2words(1211, lang='ru', to='ordinal', case='i', gender='f'),
            'тысяча двести одиннадцатой')
        self.assertEqual(
            num2words(5121000, lang='ru', to='ordinal', case='i'),
            'пять миллионов стодвадцатиоднотысячным')

    def test_to_currency(self):
        self.assertEqual(
            num2words(1.0, lang='ru', to='currency', currency='EUR'),
            'один евро, ноль центов'
        )
        self.assertEqual(
            num2words(1.0, lang='ru', to='currency', currency='RUB'),
            'один рубль, ноль копеек'
        )
        self.assertEqual(
            num2words(1.0, lang='ru', to='currency', currency='UAH'),
            'одна гривна, ноль копеек'
        )
        self.assertEqual(
            num2words(1234.56, lang='ru', to='currency', currency='EUR'),
            'одна тысяча двести тридцать четыре евро, пятьдесят шесть центов'
        )
        self.assertEqual(
            num2words(1234.56, lang='ru', to='currency', currency='RUB'),
            'одна тысяча двести тридцать четыре рубля, пятьдесят шесть копеек'
        )
        self.assertEqual(
            num2words(1234.56, lang='ru', to='currency', currency='UAH'),
            'одна тысяча двести тридцать четыре гривны, пятьдесят шесть копеек'
        )
        self.assertEqual(
            num2words(10111, lang='ru', to='currency', currency='EUR',
                      separator=' и'),
            'сто один евро и одиннадцать центов'
        )
        self.assertEqual(
            num2words(10111, lang='ru', to='currency', currency='RUB',
                      separator=' и'),
            'сто один рубль и одиннадцать копеек'
        )
        self.assertEqual(
            num2words(10111, lang='ru', to='currency', currency='UAH',
                      separator=' и'),
            'сто одна гривна и одиннадцать копеек'
        )
        self.assertEqual(
            num2words(10121, lang='ru', to='currency', currency='EUR',
                      separator=' и'),
            'сто один евро и двадцать один цент'
        )
        self.assertEqual(
            num2words(10121, lang='ru', to='currency', currency='RUB',
                      separator=' и'),
            'сто один рубль и двадцать одна копейка'
        )
        self.assertEqual(
            num2words(10121, lang='ru', to='currency', currency='UAH',
                      separator=' и'),
            'сто одна гривна и двадцать одна копейка'
        )
        self.assertEqual(
            num2words(10122, lang='ru', to='currency', currency='EUR',
                      separator=' и'),
            'сто один евро и двадцать два цента'
        )
        self.assertEqual(
            num2words(10122, lang='ru', to='currency', currency='RUB',
                      separator=' и'),
            'сто один рубль и двадцать две копейки'
        )
        self.assertEqual(
            num2words(10122, lang='ru', to='currency', currency='UAH',
                      separator=' и'),
            'сто одна гривна и двадцать две копейки'
        )
        self.assertEqual(
            num2words(-1251985, lang='ru', to='currency', currency='EUR',
                      cents=False),
            'минус двенадцать тысяч пятьсот девятнадцать евро, 85 центов'
        )
        self.assertEqual(
            num2words(-1251985, lang='ru', to='currency', currency='RUB',
                      cents=False),
            'минус двенадцать тысяч пятьсот девятнадцать рублей, 85 копеек'
        )
        self.assertEqual(
            num2words(-1251985, lang='ru', to='currency', currency='UAH',
                      cents=False),
            'минус двенадцать тысяч пятьсот девятнадцать гривен, 85 копеек'
        )
        self.assertEqual(
            num2words('38.4', lang='ru', to='currency', separator=' и',
                      cents=False, currency='EUR'),
            "тридцать восемь евро и 40 центов"
        )
        self.assertEqual(
            num2words('38.4', lang='ru', to='currency', separator=' и',
                      cents=False, currency='RUB'),
            "тридцать восемь рублей и 40 копеек"
        )
        self.assertEqual(
            num2words('38.4', lang='ru', to='currency', separator=' и',
                      cents=False, currency='UAH'),
            "тридцать восемь гривен и 40 копеек"
        )
        self.assertEqual(
            num2words('1230.56', lang='ru', to='currency', currency='USD'),
            'одна тысяча двести тридцать долларов, пятьдесят шесть центов'
        )
        self.assertEqual(
            num2words('1231.56', lang='ru', to='currency', currency='USD'),
            'одна тысяча двести тридцать один доллар, пятьдесят шесть центов'
        )
        self.assertEqual(
            num2words('1234.56', lang='ru', to='currency', currency='USD'),
            'одна тысяча двести тридцать четыре доллара, пятьдесят шесть '
            'центов'
        )
        self.assertEqual(
            num2words(10122, lang='ru', to='currency', currency='UZS',
                      separator=' и'),
            'сто один сум и двадцать два тийина'
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import division, print_function, unicode_literals

from . import lang_EU


class Num2Word_RO(lang_EU.Num2Word_EU):
    GIGA_SUFFIX = "iliard/e"
    MEGA_SUFFIX = "ilion"
    # inflection for mi/billion follows different rule
    MEGA_SUFFIX_I = "ilioane"
    GIGA_SUFFIX_I = "iliarde"

    def setup(self):
        super(Num2Word_RO, self).setup()

        self.negword = "minus "
        self.pointword = "virgulă"
        self.exclude_title = ["și", "virgulă", "minus"]
        self.errmsg_toobig = (
            "Numărul e prea mare pentru a \
fi convertit în cuvinte (abs(%s) > %s)."
        )
        self.mid_numwords = [(1000, "mie/i"), (100, "sută/e"),
                             (90, "nouăzeci"), (80, "optzeci"),
                             (70, "șaptezeci"), (60, "șaizeci"),
                             (50, "cincizeci"), (40, "patruzeci"),
                             (30, "treizeci")]
        self.low_numwords = ["douăzeci", "nouăsprezece", "optsprezece",
                             "șaptesprezece", "șaisprezece", "cincisprezece",
                             "paisprezece", "treisprezece", "doisprezece",
                             "unsprezece", "zece", "nouă", "opt", "șapte",
                             "șase", "cinci", "patru", "trei", "doi",
                             "unu", "zero"]
        self.gen_numwords = ["", "o", "două", "trei", "patru", "cinci",
                             "șase", "șapte", "opt", "nouă"]
        self.gen_numwords_n = ["", "un", "două", "trei", "patru", "cinci",
                               "șase", "șapte", "opt", "nouă"]
        self.numwords_inflections = {
            100: self.gen_numwords,
            1000: self.gen_numwords,
            1000000: self.gen_numwords_n,
            1000000000: self.gen_numwords_n
        }
        self.ords = {"unu": "primul",
                     "doi": "al doilea",
                     "three": "al treilea",
                     "cinci": "al cincilea",
                     "opt": "al optulea",
                     "nouă": "al nouălea",
                     "doisprezece": "al doisprezecelea"}

    def merge(self, lpair, rpair):
        ltext, lnum = lpair
        rtext, rnum = rpair
        rtext_i = self.inflect(rnum, rtext, lnum)
        if 1 <= lnum < 10:
            if rnum not in self.numwords_inflections:
                return (rtext, rnum)
            else:
                rtext_i = self.inflect(lnum * rnum, rtext, lnum)
                lresult = (self.numwords_inflections[rnum][lnum], rtext_i)
                return ("%s %s" % lresult, rnum)
        elif 10 < lnum < 100:
            if lnum % 10 == 0:
                if rnum in self.numwords_inflections:
                    rtext_i = self.inflect(lnum * rnum, rtext, lnum)
                    return ("%s %s" % (ltext, rtext_i), lnum * rnum)
                else:
                    return ("%s și %s" % (ltext, rtext), lnum + rnum)
            else:
                rtext_i = self.inflect(lnum * rnum, rtext, lnum)
                ltext_i = ltext if lnum % 10 != 2 \
                    else ltext.replace("doi", "două")
                return ("%s %s" % (ltext_i, rtext_i), lnum * rnum)
        else:
            if rnum in self.numwords_inflections:
                rtext_i = self.inflect(lnum * rnum, rtext, lnum)
            return ("%s %s" % (ltext, rtext_i), lnum * rnum)

    def to_ordinal(self, value):
        self.verify_ordinal(value)
        if value == 1:
            return "primul"
        else:
            value = self.to_cardinal(value)
        return "al %slea" % (value)

    def to_ordinal_num(self, value):
        self.verify_ordinal(value)
        if value == 1:
            return "1-ul"
        return "al %s-lea" % (value)

    def pluralize(self, n, forms):
        if n == 1:
            form = 0
        elif n == 0 or (n % 100 > 0 and n % 100 < 20):
            form = 1
        else:
            form = 2
        return forms[form]

    def inflect(self, value, text, side_effect=-1):
        text = text.split("/")
        result = text[0]
        if len(text) > 1:
            forms = [
                text[0],
                text[0][:-1] + text[1],
                "de " + text[0][:-1] + text[1]
            ]
            result = self.pluralize(side_effect, forms)
        # mega inflections are different
        if side_effect > 1 and result.endswith(self.MEGA_SUFFIX):
            result = result.replace(self.MEGA_SUFFIX, self.MEGA_SUFFIX_I)
        elif side_effect > 1 and result.endswith("iliare"):
            result = result.replace("iliare", self.GIGA_SUFFIX_I)
        return result

    def to_currency(self, val, currency="RON", cents=False, separator=" și",
                    adjective=False):
        # romanian currency has a particularity for numeral: one
        self.gen_numwords[1] = "una"
        result = super(Num2Word_RO, self).to_currency(
            int(round(val*100)),
            currency,
            True,
            separator,
            adjective
        )
        self.gen_numwords[1] = "o"  # revert numeral
        return result.replace(
            "unu leu", "un leu"
        ).replace(
            "unu ban", "un ban"
        ).replace(
            # if the romanian low text is 0, it is not usually printed
            separator + " zero bani", ""
        )

    def to_year(self, val, suffix=None, longval=True):
        result = super(Num2Word_RO, self).to_year(
            val,
            longval=longval
        )
        # for years we want the era negation e.g. B.C., in our case
        # it's î.Hr. or î.e.n
        if result.startswith(self.negword):
            result = result.replace(self.negword, "")
            suffix = "î.Hr." if not suffix else suffix
        if suffix:
            result = "".join([
                result,
                " ",
                suffix
            ])
        return result



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA
from __future__ import unicode_literals

from num2words import num2words

from . import test_es

TEST_CASES_TO_CURRENCY = (
    (1, 'un bolívar'),
    (2, 'dos bolívares'),
    (8, 'ocho bolívares'),
    (12, 'doce bolívares'),
    (21, 'veintiun bolívares'),
    (81.25, 'ochenta y un bolívares y veinticinco centavos'),
    (100, 'cien bolívares'),
)


class Num2WordsESVETest(test_es.Num2WordsESTest):

    def test_number(self):
        for test in test_es.TEST_CASES_CARDINAL:
            self.assertEqual(num2words(test[0], lang='es_VE'), test[1])

    def test_ordinal(self):
        for test in test_es.TEST_CASES_ORDINAL:
            self.assertEqual(
                num2words(test[0], lang='es_VE', ordinal=True),
                test[1]
            )

    def test_ordinal_num(self):
        for test in test_es.TEST_CASES_ORDINAL_NUM:
            self.assertEqual(
                num2words(test[0], lang='es', to='ordinal_num'),
                test[1]
            )

    def test_currency(self):
        for test in TEST_CASES_TO_CURRENCY:
            self.assertEqual(
                num2words(test[0], lang='es_VE', to='currency', old=True),
                test[1]
            )



# coding: utf-8
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsISTest(TestCase):

    def test_cardinal(self):
        self.assertEqual(num2words(0, to="cardinal", lang="is"),
                         "núll")
        self.assertEqual(num2words(1, to="cardinal", lang="is"),
                         "einn")
        self.assertEqual(num2words(45, to="cardinal", lang="is"),
                         "fjörutíu og fimm")
        self.assertEqual(num2words(145, to="cardinal", lang="is"),
                         "eitt hundrað fjörutíu og fimm")
        self.assertEqual(num2words(-1245, to="cardinal", lang="is"),
                         "mínus eitt þúsund tvö hundruð fjörutíu og fimm")
        self.assertEqual(num2words(2234045, to="cardinal", lang="is"),
                         "tvær milljónir tvö hundruð þrjátíu og fjögur þúsund "
                         "fjörutíu og fimm")
        self.assertEqual(num2words(4002234045, to="cardinal", lang="is"),
                         "fjórir milljarðar tvær milljónir tvö hundruð "
                         "þrjátíu og fjögur þúsund fjörutíu og fimm")

    def test_cardinal_for_float_number(self):
        self.assertEqual(num2words(12.5, to="cardinal", lang="is"),
                         "tólf komma fimm")
        self.assertEqual(num2words(12.51, to="cardinal", lang="is"),
                         "tólf komma fimm einn")
        self.assertEqual(num2words(-12.53, to="cardinal", lang="is"),
                         "mínus tólf komma fimm þrír")
        self.assertEqual(num2words(12.59, to="cardinal", lang="is"),
                         "tólf komma fimm níu")

    def test_overflow(self):
        with self.assertRaises(OverflowError):
            num2words("1000000000000000000000000000000000000000000000000000000"
                      "0000000000000000000000000000000000000000000000000000000"
                      "0000000000000000000000000000000000000000000000000000000"
                      "0000000000000000000000000000000000000000000000000000000"
                      "0000000000000000000000000000000000000000000000000000000"
                      "00000000000000000000000000000000", lang="is")

    def test_not_implemented(self):
        # Ordinals
        with self.assertRaises(NotImplementedError):
            num2words(1, to="ordinal", lang="is")

        # Ordinal num
        with self.assertRaises(NotImplementedError):
            num2words(1, to="ordinal_num", lang="is")

        # Year
        with self.assertRaises(NotImplementedError):
            num2words(1, to="year", lang="is")

        # Currency
        with self.assertRaises(NotImplementedError):
            num2words(1, to="currency", lang="is")



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from unittest import TestCase

from num2words.utils import splitbyx


class TestUtils(TestCase):
    def test_splitbyx(self):
        self.assertEqual(list(splitbyx(str(12), 3)), [12])
        self.assertEqual(list(splitbyx(str(1234), 3)), [1, 234])
        self.assertEqual(list(splitbyx(str(12345678900), 3)),
                         [12, 345, 678, 900]
                         )
        self.assertEqual(list(splitbyx(str(1000000), 6)), [1, 0])

        self.assertEqual(list(splitbyx(str(12), 3, format_int=False)), ['12'])
        self.assertEqual(list(splitbyx(str(1234), 3, format_int=False)),
                         ['1', '234']
                         )
        self.assertEqual(list(splitbyx(str(12345678900), 3, format_int=False)),
                         ['12', '345', '678', '900']
                         )
        self.assertEqual(list(splitbyx(str(1000000), 6, format_int=False)),
                         ['1', '000000']
                         )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsLTTest(TestCase):
    def test_to_cardinal(self):
        self.assertEqual(num2words(100, lang='lt'), 'vienas šimtas')
        self.assertEqual(num2words(101, lang='lt'), 'vienas šimtas vienas')
        self.assertEqual(num2words(110, lang='lt'), 'vienas šimtas dešimt')
        self.assertEqual(num2words(115, lang='lt'),
                         'vienas šimtas penkiolika')
        self.assertEqual(num2words(123, lang='lt'),
                         'vienas šimtas dvidešimt trys')
        self.assertEqual(num2words(1000, lang='lt'), 'vienas tūkstantis')
        self.assertEqual(num2words(1001, lang='lt'),
                         'vienas tūkstantis vienas')
        self.assertEqual(num2words(2012, lang='lt'),
                         'du tūkstančiai dvylika')
        self.assertEqual(
            num2words(1234567890, lang='lt'),
            "vienas milijardas du šimtai trisdešimt keturi milijonai "
            "penki šimtai šešiasdešimt septyni tūkstančiai aštuoni šimtai "
            "devyniasdešimt")
        self.assertEqual(
            num2words(215461407892039002157189883901676, lang='lt'),
            "du šimtai penkiolika naintilijonų keturi šimtai šešiasdešimt "
            "vienas oktilijonas keturi šimtai septyni septilijonai aštuoni "
            "šimtai devyniasdešimt du sikstilijonai trisdešimt devyni "
            "kvintilijonai du kvadrilijonai vienas šimtas penkiasdešimt "
            "septyni trilijonai vienas šimtas aštuoniasdešimt devyni "
            "milijardai aštuoni šimtai aštuoniasdešimt trys milijonai "
            "devyni šimtai vienas tūkstantis šeši šimtai "
            "septyniasdešimt šeši")
        self.assertEqual(
            num2words(719094234693663034822824384220291, lang='lt'),
            "septyni šimtai devyniolika naintilijonų devyniasdešimt keturi "
            "oktilijonai du šimtai trisdešimt keturi septilijonai šeši "
            "šimtai devyniasdešimt trys sikstilijonai šeši šimtai "
            "šešiasdešimt trys kvintilijonai trisdešimt keturi kvadrilijonai "
            "aštuoni šimtai dvidešimt du trilijonai aštuoni šimtai dvidešimt "
            "keturi milijardai trys šimtai aštuoniasdešimt keturi milijonai "
            "du šimtai dvidešimt tūkstančių du šimtai devyniasdešimt vienas")
        self.assertEqual(
            num2words(-5000, lang='lt'),
            'minus penki tūkstančiai',
        )
        self.assertEqual(
            num2words(-5000.22, lang='lt'),
            'minus penki tūkstančiai kablelis dvidešimt du',
        )
        self.assertEqual(
            num2words(10.02, lang='lt'),
            "dešimt kablelis nulis du"
        )
        self.assertEqual(
            num2words(15.007, lang='lt'),
            "penkiolika kablelis nulis nulis septyni"
        )

    def test_to_ordinal(self):
        # @TODO: implement to_ordinal
        with self.assertRaises(NotImplementedError):
            num2words(1, lang='lt', to='ordinal')

    def test_to_currency(self):
        # Test all available currency forms.
        # LTL
        self.assertEqual(
            num2words(1.0, lang='lt', to='currency', currency='LTL'),
            'vienas litas, nulis centų'
        )
        self.assertEqual(
            num2words(10.01, lang='lt', to='currency', currency='LTL'),
            'dešimt litų, vienas centas'
        )
        self.assertEqual(
            num2words(1234.56, lang='lt', to='currency', currency='LTL'),
            'vienas tūkstantis du šimtai trisdešimt keturi litai, '
            'penkiasdešimt šeši centai'
        )
        # EUR
        self.assertEqual(
            num2words(-1251981, lang='lt', to='currency', currency='EUR',
                      cents=False),
            'minus dvylika tūkstančių penki šimtai devyniolika eurų, '
            '81 centas'
        )
        self.assertEqual(
            num2words(1.0, lang='lt', to='currency', currency='EUR'),
            'vienas euras, nulis centų'
        )
        self.assertEqual(
            num2words(1234.56, lang='lt', to='currency', currency='EUR'),
            'vienas tūkstantis du šimtai trisdešimt keturi eurai, '
            'penkiasdešimt šeši centai'
        )
        self.assertEqual(
            num2words(1122.22, lang='lt', to='currency', currency='EUR'),
            'vienas tūkstantis vienas šimtas dvidešimt du eurai, '
            'dvidešimt du centai'
        )
        # USD
        self.assertEqual(
            num2words(-1281, lang='lt', to='currency', currency='USD',
                      cents=False),
            'minus dvylika dolerių, 81 centas'
        )
        self.assertEqual(
            num2words(1.0, lang='lt', to='currency', currency='USD'),
            'vienas doleris, nulis centų'
        )
        self.assertEqual(
            num2words(5.06, lang='lt', to='currency', currency='USD'),
            'penki doleriai, šeši centai'
        )
        # GBP
        self.assertEqual(
            num2words(-1281, lang='lt', to='currency', currency='GBP',
                      cents=False),
            'minus dvylika svarų sterlingų, 81 pensas'
        )
        self.assertEqual(
            num2words(1.0, lang='lt', to='currency', currency='GBP'),
            'vienas svaras sterlingų, nulis pensų'
        )
        self.assertEqual(
            num2words(5.06, lang='lt', to='currency', currency='GBP'),
            'penki svarai sterlingų, šeši pensai'
        )
        # PLN
        self.assertEqual(
            num2words(-1281, lang='lt', to='currency', currency='PLN',
                      cents=False),
            'minus dvylika zlotų, 81 grašis'
        )
        self.assertEqual(
            num2words(1.0, lang='lt', to='currency', currency='PLN'),
            'vienas zlotas, nulis grašių'
        )
        self.assertEqual(
            num2words(5.06, lang='lt', to='currency', currency='PLN'),
            'penki zlotai, šeši grašiai'
        )
        # RUB
        self.assertEqual(
            num2words(-1281, lang='lt', to='currency', currency='RUB',
                      cents=False),
            'minus dvylika rublių, 81 kapeika'
        )
        self.assertEqual(
            num2words(1.0, lang='lt', to='currency', currency='RUB'),
            'vienas rublis, nulis kapeikų'
        )
        self.assertEqual(
            num2words(5.06, lang='lt', to='currency', currency='RUB'),
            'penki rubliai, šešios kapeikos'
        )
        self.assertEqual(
            num2words(-12.01, lang='lt', to='currency', currency='RUB'),
            'minus dvylika rublių, viena kapeika'
        )
        self.assertEqual(
            num2words(1122.22, lang='lt', to='currency', currency='RUB'),
            'vienas tūkstantis vienas šimtas dvidešimt du rubliai, '
            'dvidešimt dvi kapeikos'
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from decimal import Decimal
from unittest import TestCase

from num2words import num2words
from num2words.lang_PT_BR import Num2Word_PT_BR


class Num2WordsPTBRTest(TestCase):
    def setUp(self):
        super(Num2WordsPTBRTest, self).setUp()
        self.n2w = Num2Word_PT_BR()

    def test_cardinal_integer(self):
        self.assertEqual(num2words(1, lang='pt_BR'), 'um')
        self.assertEqual(num2words(2, lang='pt_BR'), 'dois')
        self.assertEqual(num2words(3, lang='pt_BR'), 'três')
        self.assertEqual(num2words(4, lang='pt_BR'), 'quatro')
        self.assertEqual(num2words(5, lang='pt_BR'), 'cinco')
        self.assertEqual(num2words(6, lang='pt_BR'), 'seis')
        self.assertEqual(num2words(7, lang='pt_BR'), 'sete')
        self.assertEqual(num2words(8, lang='pt_BR'), 'oito')
        self.assertEqual(num2words(9, lang='pt_BR'), 'nove')
        self.assertEqual(num2words(10, lang='pt_BR'), 'dez')
        self.assertEqual(num2words(11, lang='pt_BR'), 'onze')
        self.assertEqual(num2words(12, lang='pt_BR'), 'doze')
        self.assertEqual(num2words(13, lang='pt_BR'), 'treze')
        self.assertEqual(num2words(14, lang='pt_BR'), 'catorze')
        self.assertEqual(num2words(15, lang='pt_BR'), 'quinze')
        self.assertEqual(num2words(16, lang='pt_BR'), 'dezesseis')
        self.assertEqual(num2words(17, lang='pt_BR'), 'dezessete')
        self.assertEqual(num2words(18, lang='pt_BR'), 'dezoito')
        self.assertEqual(num2words(19, lang='pt_BR'), 'dezenove')
        self.assertEqual(num2words(20, lang='pt_BR'), 'vinte')

        self.assertEqual(num2words(21, lang='pt_BR'), 'vinte e um')
        self.assertEqual(num2words(22, lang='pt_BR'), 'vinte e dois')
        self.assertEqual(num2words(35, lang='pt_BR'), 'trinta e cinco')
        self.assertEqual(num2words(99, lang='pt_BR'), 'noventa e nove')

        self.assertEqual(num2words(100, lang='pt_BR'), 'cem')
        self.assertEqual(num2words(101, lang='pt_BR'), 'cento e um')
        self.assertEqual(num2words(128, lang='pt_BR'), 'cento e vinte e oito')
        self.assertEqual(num2words(713, lang='pt_BR'), 'setecentos e treze')

        self.assertEqual(num2words(1000, lang='pt_BR'), 'mil')
        self.assertEqual(num2words(1001, lang='pt_BR'), 'mil e um')
        self.assertEqual(num2words(1111, lang='pt_BR'), 'mil, cento e onze')
        self.assertEqual(
            num2words(2114, lang='pt_BR'), 'dois mil, cento e catorze'
        )
        self.assertEqual(
            num2words(73421, lang='pt_BR'),
            'setenta e três mil, quatrocentos e vinte e um'
        )

        self.assertEqual(num2words(100000, lang='pt_BR'), 'cem mil')
        self.assertEqual(
            num2words(250050, lang='pt_BR'),
            'duzentos e cinquenta mil e cinquenta'
        )
        self.assertEqual(
            num2words(6000000, lang='pt_BR'), 'seis milhões'
        )
        self.assertEqual(
            num2words(100000000, lang='pt_BR'), 'cem milhões'
        )
        self.assertEqual(
            num2words(100000000000, lang='pt_BR'), 'cem bilhões'
        )
        self.assertEqual(
            num2words(19000000000, lang='pt_BR'), 'dezenove bilhões'
        )
        self.assertEqual(
            num2words(145000000002, lang='pt_BR'),
            'cento e quarenta e cinco bilhões e dois'
        )

    def test_cardinal_integer_negative(self):
        self.assertEqual(num2words(-1, lang='pt_BR'), 'menos um')
        self.assertEqual(
            num2words(-256, lang='pt_BR'), 'menos duzentos e cinquenta e seis'
        )
        self.assertEqual(num2words(-1000, lang='pt_BR'), 'menos mil')
        self.assertEqual(num2words(-1000000, lang='pt_BR'), 'menos um milhão')
        self.assertEqual(
            num2words(-1234567, lang='pt_BR'),
            'menos um milhão, duzentos e trinta e quatro mil, quinhentos e '
            'sessenta e sete'
        )

    def test_cardinal_float(self):
        self.assertEqual(num2words(Decimal('1.00'), lang='pt_BR'), 'um')
        self.assertEqual(num2words(
            Decimal('1.01'), lang='pt_BR'), 'um vírgula zero um')
        self.assertEqual(num2words(
            Decimal('1.035'), lang='pt_BR'), 'um vírgula zero três cinco'
        )
        self.assertEqual(num2words(
            Decimal('1.35'), lang='pt_BR'), 'um vírgula três cinco'
        )
        self.assertEqual(
            num2words(Decimal('3.14159'), lang='pt_BR'),
            'três vírgula um quatro um cinco nove'
        )
        self.assertEqual(
            num2words(Decimal('101.22'), lang='pt_BR'),
            'cento e um vírgula dois dois'
        )
        self.assertEqual(
            num2words(Decimal('2345.75'), lang='pt_BR'),
            'dois mil, trezentos e quarenta e cinco vírgula sete cinco')

    def test_cardinal_float_negative(self):
        self.assertEqual(
            num2words(Decimal('-2.34'), lang='pt_BR'),
            'menos dois vírgula três quatro'
        )
        self.assertEqual(
            num2words(Decimal('-9.99'), lang='pt_BR'),
            'menos nove vírgula nove nove'
        )
        self.assertEqual(
            num2words(Decimal('-7.01'), lang='pt_BR'),
            'menos sete vírgula zero um'
        )
        self.assertEqual(
            num2words(Decimal('-222.22'), lang='pt_BR'),
            'menos duzentos e vinte e dois vírgula dois dois'
        )

    def test_ordinal(self):
        self.assertEqual(num2words(1, lang='pt_BR', ordinal=True), 'primeiro')
        self.assertEqual(num2words(2, lang='pt_BR', ordinal=True), 'segundo')
        self.assertEqual(num2words(3, lang='pt_BR', ordinal=True), 'terceiro')
        self.assertEqual(num2words(4, lang='pt_BR', ordinal=True), 'quarto')
        self.assertEqual(num2words(5, lang='pt_BR', ordinal=True), 'quinto')
        self.assertEqual(num2words(6, lang='pt_BR', ordinal=True), 'sexto')
        self.assertEqual(num2words(7, lang='pt_BR', ordinal=True), 'sétimo')
        self.assertEqual(num2words(8, lang='pt_BR', ordinal=True), 'oitavo')
        self.assertEqual(num2words(9, lang='pt_BR', ordinal=True), 'nono')
        self.assertEqual(num2words(10, lang='pt_BR', ordinal=True), 'décimo')
        self.assertEqual(
            num2words(11, lang='pt_BR', ordinal=True), 'décimo primeiro'
        )
        self.assertEqual(
            num2words(12, lang='pt_BR', ordinal=True), 'décimo segundo'
        )
        self.assertEqual(
            num2words(13, lang='pt_BR', ordinal=True), 'décimo terceiro'
        )
        self.assertEqual(
            num2words(14, lang='pt_BR', ordinal=True), 'décimo quarto'
        )
        self.assertEqual(
            num2words(15, lang='pt_BR', ordinal=True), 'décimo quinto'
        )
        self.assertEqual(
            num2words(16, lang='pt_BR', ordinal=True), 'décimo sexto'
        )
        self.assertEqual(
            num2words(17, lang='pt_BR', ordinal=True), 'décimo sétimo'
        )
        self.assertEqual(
            num2words(18, lang='pt_BR', ordinal=True), 'décimo oitavo'
        )
        self.assertEqual(
            num2words(19, lang='pt_BR', ordinal=True), 'décimo nono'
        )
        self.assertEqual(
            num2words(20, lang='pt_BR', ordinal=True), 'vigésimo'
        )

        self.assertEqual(
            num2words(21, lang='pt_BR', ordinal=True), 'vigésimo primeiro'
        )
        self.assertEqual(
            num2words(22, lang='pt_BR', ordinal=True), 'vigésimo segundo'
        )
        self.assertEqual(
            num2words(35, lang='pt_BR', ordinal=True), 'trigésimo quinto'
        )
        self.assertEqual(
            num2words(99, lang='pt_BR', ordinal=True), 'nonagésimo nono'
        )

        self.assertEqual(
            num2words(100, lang='pt_BR', ordinal=True), 'centésimo'
        )
        self.assertEqual(
            num2words(101, lang='pt_BR', ordinal=True), 'centésimo primeiro'
        )
        self.assertEqual(
            num2words(128, lang='pt_BR', ordinal=True),
            'centésimo vigésimo oitavo'
        )
        self.assertEqual(
            num2words(713, lang='pt_BR', ordinal=True),
            'septigentésimo décimo terceiro'
        )

        self.assertEqual(
            num2words(1000, lang='pt_BR', ordinal=True), 'milésimo'
        )
        self.assertEqual(
            num2words(1001, lang='pt_BR', ordinal=True), 'milésimo primeiro'
        )
        self.assertEqual(
            num2words(1111, lang='pt_BR', ordinal=True),
            'milésimo centésimo décimo primeiro'
        )
        self.assertEqual(
            num2words(2114, lang='pt_BR', ordinal=True),
            'segundo milésimo centésimo décimo quarto'
        )
        self.assertEqual(
            num2words(73421, lang='pt_BR', ordinal=True),
            'septuagésimo terceiro milésimo quadrigentésimo vigésimo primeiro'
        )

        self.assertEqual(
            num2words(100000, lang='pt_BR', ordinal=True),
            'centésimo milésimo'
        )
        self.assertEqual(
            num2words(250050, lang='pt_BR', ordinal=True),
            'ducentésimo quinquagésimo milésimo quinquagésimo'
        )
        self.assertEqual(
            num2words(6000000, lang='pt_BR', ordinal=True), 'sexto milionésimo'
        )
        self.assertEqual(
            num2words(19000000000, lang='pt_BR', ordinal=True),
            'décimo nono bilionésimo'
        )
        self.assertEqual(
            num2words(145000000002, lang='pt_BR', ordinal=True),
            'centésimo quadragésimo quinto bilionésimo segundo'
        )

    def test_currency_integer(self):
        self.assertEqual(self.n2w.to_currency(1), 'um real')
        self.assertEqual(self.n2w.to_currency(2), 'dois reais')
        self.assertEqual(self.n2w.to_currency(3), 'três reais')
        self.assertEqual(self.n2w.to_currency(4), 'quatro reais')
        self.assertEqual(self.n2w.to_currency(5), 'cinco reais')
        self.assertEqual(self.n2w.to_currency(6), 'seis reais')
        self.assertEqual(self.n2w.to_currency(7), 'sete reais')
        self.assertEqual(self.n2w.to_currency(8), 'oito reais')
        self.assertEqual(self.n2w.to_currency(9), 'nove reais')
        self.assertEqual(self.n2w.to_currency(10), 'dez reais')
        self.assertEqual(self.n2w.to_currency(11), 'onze reais')
        self.assertEqual(self.n2w.to_currency(12), 'doze reais')
        self.assertEqual(self.n2w.to_currency(13), 'treze reais')
        self.assertEqual(self.n2w.to_currency(14), 'catorze reais')
        self.assertEqual(self.n2w.to_currency(15), 'quinze reais')
        self.assertEqual(self.n2w.to_currency(16), 'dezesseis reais')
        self.assertEqual(self.n2w.to_currency(17), 'dezessete reais')
        self.assertEqual(self.n2w.to_currency(18), 'dezoito reais')
        self.assertEqual(self.n2w.to_currency(19), 'dezenove reais')
        self.assertEqual(self.n2w.to_currency(20), 'vinte reais')

        self.assertEqual(self.n2w.to_currency(21), 'vinte e um reais')
        self.assertEqual(self.n2w.to_currency(22), 'vinte e dois reais')
        self.assertEqual(self.n2w.to_currency(35), 'trinta e cinco reais')
        self.assertEqual(self.n2w.to_currency(99), 'noventa e nove reais')

        self.assertEqual(self.n2w.to_currency(100), 'cem reais')
        self.assertEqual(self.n2w.to_currency(101), 'cento e um reais')
        self.assertEqual(
            self.n2w.to_currency(128), 'cento e vinte e oito reais'
        )
        self.assertEqual(self.n2w.to_currency(713), 'setecentos e treze reais')

        self.assertEqual(self.n2w.to_currency(1000), 'mil reais')
        self.assertEqual(self.n2w.to_currency(1001), 'mil e um reais')
        self.assertEqual(self.n2w.to_currency(1111), 'mil, cento e onze reais')
        self.assertEqual(
            self.n2w.to_currency(2114), 'dois mil, cento e catorze reais'
        )
        self.assertEqual(
            self.n2w.to_currency(73421),
            'setenta e três mil, quatrocentos e vinte e um reais'
        )

        self.assertEqual(self.n2w.to_currency(100000), 'cem mil reais')
        self.assertEqual(
            self.n2w.to_currency(250050),
            'duzentos e cinquenta mil e cinquenta reais'
        )
        self.assertEqual(
            self.n2w.to_currency(6000000), 'seis milhões de reais'
        )
        self.assertEqual(
            self.n2w.to_currency(19000000000), 'dezenove bilhões de reais'
        )
        self.assertEqual(
            self.n2w.to_currency(145000000002),
            'cento e quarenta e cinco bilhões e dois reais'
        )

    def test_currency_integer_negative(self):
        self.assertEqual(self.n2w.to_currency(-1), 'menos um real')
        self.assertEqual(
            self.n2w.to_currency(-256),
            'menos duzentos e cinquenta e seis reais'
        )
        self.assertEqual(self.n2w.to_currency(-1000), 'menos mil reais')
        self.assertEqual(
            self.n2w.to_currency(-1000000), 'menos um milhão de reais'
        )
        self.assertEqual(
            self.n2w.to_currency(-1234567),
            'menos um milhão, duzentos e trinta e quatro mil, quinhentos e '
            'sessenta e sete reais'
        )

    def test_currency_float(self):
        self.assertEqual(self.n2w.to_currency(Decimal('1.00')), 'um real')
        self.assertEqual(
            self.n2w.to_currency(Decimal('1.01')), 'um real e um centavo'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('1.03')), 'um real e três centavos'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('1.35')),
            'um real e trinta e cinco centavos'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('3.14')),
            'três reais e catorze centavos'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('101.22')),
            'cento e um reais e vinte e dois centavos'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('2345.75')),
            'dois mil, trezentos e quarenta e cinco reais e setenta e cinco '
            'centavos'
        )

    def test_currency_float_negative(self):
        self.assertEqual(
            self.n2w.to_currency(Decimal('-2.34')),
            'menos dois reais e trinta e quatro centavos'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('-9.99')),
            'menos nove reais e noventa e nove centavos'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('-7.01')),
            'menos sete reais e um centavo'
        )
        self.assertEqual(
            self.n2w.to_currency(Decimal('-222.22')),
            'menos duzentos e vinte e dois reais e vinte e dois centavos'
        )

    def test_year(self):
        self.assertEqual(self.n2w.to_year(1001), 'mil e um')
        self.assertEqual(
            self.n2w.to_year(1789), 'mil, setecentos e oitenta e nove'
        )
        self.assertEqual(
            self.n2w.to_year(1942), 'mil, novecentos e quarenta e dois'
        )
        self.assertEqual(
            self.n2w.to_year(1984), 'mil, novecentos e oitenta e quatro'
        )
        self.assertEqual(self.n2w.to_year(2000), 'dois mil')
        self.assertEqual(self.n2w.to_year(2001), 'dois mil e um')
        self.assertEqual(self.n2w.to_year(2016), 'dois mil e dezesseis')

    def test_year_negative(self):
        self.assertEqual(self.n2w.to_year(-30), 'trinta antes de Cristo')
        self.assertEqual(
            self.n2w.to_year(-744),
            'setecentos e quarenta e quatro antes de Cristo'
        )
        self.assertEqual(self.n2w.to_year(-10000), 'dez mil antes de Cristo')



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.
# Copyright (c) 2023, Sergei Ruzki/Ivan Shakh  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsBYTest(TestCase):
    def test_cardinal(self):
        self.assertEqual(num2words(100, lang="by"), "сто")
        self.assertEqual(num2words(101, lang="by"), "сто адзін")
        self.assertEqual(num2words(110, lang="by"), "сто дзесяць")
        self.assertEqual(num2words(115, lang="by"), "сто пятнаццаць")
        self.assertEqual(num2words(123, lang="by"), "сто дваццаць тры")
        self.assertEqual(num2words(1000, lang="by"), "адна тысяча")
        self.assertEqual(num2words(1001, lang="by"), "адна тысяча адзін")
        self.assertEqual(num2words(2012, lang="by"), "дзве тысячы дванаццаць")
        self.assertEqual(
            num2words(12519.85, lang="by"),
            "дванаццаць тысяч пяцьсот дзевятнаццаць коска восемдзесят пяць",
        )
        self.assertEqual(
            num2words(1234567890, lang="by"),
            "адзін мільярд дзвесце трыццаць чатыры мільёны пяцьсот "
            "шэсцьдзясят сем тысяч восемсот дзевяноста",
        )
        self.assertEqual(
            num2words(461407892039002157189883901676, lang="by"),
            "чатырыста шэсцьдзясят адзін "
            "актыльён чатырыста сем сэптыльёнаў восемсот дзевяноста "
            "два секстыльёны трыццаць дзевяць квінтыльёнаў два квадрыльёны "
            "сто пяцьдзясят сем трыльёнаў сто восемдзесят дзевяць мільярдаў "
            "восемсот восемдзесят тры мільёны дзевяцьсот адна тысяча "
            "шэсцьсот семдзесят шэсць",
        )
        self.assertEqual(
            num2words(94234693663034822824384220291, lang="by"),
            "дзевяноста чатыры актыльёны "
            "дзвесце трыццаць чатыры сэптыльёны шэсцьсот дзевяноста тры "
            "секстыльёны шэсцьсот шэсцьдзясят тры квінтыльёны трыццаць "
            "чатыры квадрыльёны восемсот дваццаць два трыльёны восемсот "
            "дваццаць чатыры мільярды трыста восемдзесят чатыры мільёны "
            "дзвесце дваццаць тысяч дзвесце дзевяноста адзін",
        )
        self.assertEqual(num2words(5, lang="by"), "пяць")
        self.assertEqual(num2words(15, lang="by"), "пятнаццаць")
        self.assertEqual(num2words(154, lang="by"), "сто пяцьдзясят чатыры")
        self.assertEqual(
            num2words(1135, lang="by"), "адна тысяча сто трыццаць пяць"
        )
        self.assertEqual(
            num2words(418531, lang="by"),
            "чатырыста васямнаццаць тысяч пяцьсот трыццаць адзін",
        )
        self.assertEqual(
            num2words(1000139, lang="by"), "адзін мільён сто трыццаць дзевяць"
        )
        self.assertEqual(num2words(-1, lang="by"), "мінус адзін")
        self.assertEqual(num2words(-15, lang="by"), "мінус пятнаццаць")
        self.assertEqual(num2words(-100, lang="by"), "мінус сто")

    def test_floating_point(self):
        self.assertEqual(num2words(5.2, lang="by"), "пяць коска два")
        self.assertEqual(num2words(10.02, lang="by"), "дзесяць коска нуль два")
        self.assertEqual(
            num2words(15.007, lang="by"), "пятнаццаць коска нуль нуль сем"
        )
        self.assertEqual(
            num2words(561.42, lang="by"),
            "пяцьсот шэсцьдзясят адзін коска сорак два",
        )

        self.assertEqual(
            num2words(561.0, lang="by"), "пяцьсот шэсцьдзясят адзін коска нуль"
        )

    def test_to_ordinal(self):
        self.assertEqual(num2words(1, lang="by", to="ordinal"), "першы")
        self.assertEqual(num2words(5, lang="by", to="ordinal"), "пяты")
        self.assertEqual(num2words(6, lang="by", to="ordinal"), "шосты")
        self.assertEqual(num2words(10, lang="by", to="ordinal"), "дзясяты")

        self.assertEqual(num2words(13, lang="by", to="ordinal"), "трынаццаты")
        self.assertEqual(num2words(20, lang="by", to="ordinal"), "дваццаты")
        self.assertEqual(
            num2words(23, lang="by", to="ordinal"), "дваццаць трэці"
        )
        self.assertEqual(
            num2words(23, lang="by", to="ordinal", gender="f"),
            "дваццаць трэцяя",
        )
        self.assertEqual(
            num2words(23, lang="by", to="ordinal", gender="n"),
            "дваццаць трэцяе",
        )
        self.assertEqual(num2words(40, lang="by", to="ordinal"), "саракавы")
        self.assertEqual(
            num2words(61, lang="by", to="ordinal"), "шэсцьдзясят першы"
        )
        self.assertEqual(num2words(70, lang="by", to="ordinal"), "сямідзясяты")
        self.assertEqual(num2words(100, lang="by", to="ordinal"), "соты")
        self.assertEqual(
            num2words(136, lang="by", to="ordinal"), "сто трыццаць шосты"
        )
        self.assertEqual(num2words(500, lang="by", to="ordinal"), "пяцісоты")

        self.assertEqual(
            num2words(500, lang="by", to="ordinal", gender="f"), "пяцісотая"
        )

        self.assertEqual(
            num2words(500, lang="by", to="ordinal", gender="n"), "пяцісотае"
        )

        self.assertEqual(num2words(1000, lang="by", to="ordinal"), "тысячны")

        self.assertEqual(
            num2words(1000, lang="by", to="ordinal", gender="f"), "тысячная"
        )

        self.assertEqual(
            num2words(1000, lang="by", to="ordinal", gender="n"), "тысячнае"
        )

        self.assertEqual(
            num2words(1001, lang="by", to="ordinal"), "тысяча першы"
        )
        self.assertEqual(
            num2words(2000, lang="by", to="ordinal"), "двухтысячны"
        )
        self.assertEqual(
            num2words(10000, lang="by", to="ordinal"), "дзесяцітысячны"
        )
        self.assertEqual(
            num2words(42000, lang="by", to="ordinal"), "саракадвухтысячны"
        )
        self.assertEqual(
            num2words(1000000, lang="by", to="ordinal"), "мільённы"
        )
        self.assertEqual(
            num2words(1000000000, lang="by", to="ordinal"), "мільярдны"
        )

    def test_to_currency(self):
        self.assertEqual(
            num2words(1.0, lang="by", to="currency", currency="EUR"),
            "адзін еўра, нуль цэнтаў",
        )
        self.assertEqual(
            num2words(1.0, lang="by", to="currency", currency="RUB"),
            "адзін расійскі рубель, нуль капеек",
        )
        self.assertEqual(
            num2words(1.0, lang="by", to="currency", currency="BYN"),
            "адзін беларускі рубель, нуль капеек",
        )
        self.assertEqual(
            num2words(1.0, lang="by", to="currency", currency="UAH"),
            "адна грыўна, нуль капеек",
        )
        self.assertEqual(
            num2words(1234.56, lang="by", to="currency", currency="EUR"),
            "адна тысяча дзвесце трыццаць чатыры еўра, "
            "пяцьдзясят шэсць цэнтаў",
        )
        self.assertEqual(
            num2words(1234.56, lang="by", to="currency", currency="RUB"),
            "адна тысяча дзвесце трыццаць чатыры расійскія рублі, "
            "пяцьдзясят шэсць капеек",
        )
        self.assertEqual(
            num2words(1234.56, lang="by", to="currency", currency="BYN"),
            "адна тысяча дзвесце трыццаць чатыры беларускія рублі, "
            "пяцьдзясят шэсць капеек",
        )
        self.assertEqual(
            num2words(1234.56, lang="by", to="currency", currency="UAH"),
            "адна тысяча дзвесце трыццаць чатыры грыўны, "
            "пяцьдзясят шэсць капеек",
        )
        self.assertEqual(
            num2words(
                10111, lang="by", to="currency", currency="EUR", separator=" і"
            ),
            "сто адзін еўра і адзінаццаць цэнтаў",
        )
        self.assertEqual(
            num2words(
                10111, lang="by", to="currency", currency="RUB", separator=" і"
            ),
            "сто адзін расійскі рубель і адзінаццаць капеек",
        )
        self.assertEqual(
            num2words(
                10111, lang="by", to="currency", currency="BYN", separator=" і"
            ),
            "сто адзін беларускі рубель і адзінаццаць капеек",
        )
        self.assertEqual(
            num2words(
                10111, lang="by", to="currency", currency="UAH", separator=" і"
            ),
            "сто адна грыўна і адзінаццаць капеек",
        )
        self.assertEqual(
            num2words(
                10121, lang="by", to="currency", currency="EUR", separator=" і"
            ),
            "сто адзін еўра і дваццаць адзін цэнт",
        )
        self.assertEqual(
            num2words(
                10121, lang="by", to="currency", currency="RUB", separator=" і"
            ),
            "сто адзін расійскі рубель і дваццаць адна капейка",
        )
        self.assertEqual(
            num2words(
                10121, lang="by", to="currency", currency="BYN", separator=" і"
            ),
            "сто адзін беларускі рубель і дваццаць адна капейка",
        )
        self.assertEqual(
            num2words(
                10121, lang="by", to="currency", currency="UAH", separator=" і"
            ),
            "сто адна грыўна і дваццаць адна капейка",
        )
        self.assertEqual(
            num2words(
                10122, lang="by", to="currency", currency="EUR", separator=" і"
            ),
            "сто адзін еўра і дваццаць два цэнты",
        )
        self.assertEqual(
            num2words(
                10122, lang="by", to="currency", currency="RUB", separator=" і"
            ),
            "сто адзін расійскі рубель і дваццаць дзве капейкі",
        )
        self.assertEqual(
            num2words(
                10122, lang="by", to="currency", currency="BYN", separator=" і"
            ),
            "сто адзін беларускі рубель і дваццаць дзве капейкі",
        )
        self.assertEqual(
            num2words(
                10122, lang="by", to="currency", currency="UAH", separator=" і"
            ),
            "сто адна грыўна і дваццаць дзве капейкі",
        )
        self.assertEqual(
            num2words(
                10122, lang="by", to="currency", currency="KZT", separator=" і"
            ),
            "сто адзін тэнге і дваццаць два тыйіны",
        )
        self.assertEqual(
            num2words(
                -1251985, lang="by", to="currency", currency="EUR", cents=False
            ),
            "мінус дванаццаць тысяч пяцьсот дзевятнаццаць еўра, 85 цэнтаў",
        )
        self.assertEqual(
            num2words(
                -1251985, lang="by", to="currency", currency="RUB", cents=False
            ),
            "мінус дванаццаць тысяч пяцьсот дзевятнаццаць "
            "расійскіх рублёў, 85 капеек",
        )
        self.assertEqual(
            num2words(
                -1251985, lang="by", to="currency", currency="BYN", cents=False
            ),
            "мінус дванаццаць тысяч пяцьсот дзевятнаццаць "
            "беларускіх рублёў, 85 капеек",
        )
        self.assertEqual(
            num2words(
                -1251985, lang="by", to="currency", currency="UAH", cents=False
            ),
            "мінус дванаццаць тысяч пяцьсот дзевятнаццаць грыўнаў, 85 капеек",
        )
        self.assertEqual(
            num2words(
                "38.4",
                lang="by",
                to="currency",
                separator=" і",
                cents=False,
                currency="EUR",
            ),
            "трыццаць восем еўра і 40 цэнтаў",
        )
        self.assertEqual(
            num2words(
                "38.4",
                lang="by",
                to="currency",
                separator=" і",
                cents=False,
                currency="RUB",
            ),
            "трыццаць восем расійскіх рублёў і 40 капеек",
        )
        self.assertEqual(
            num2words(
                "38.4",
                lang="by",
                to="currency",
                separator=" і",
                cents=False,
                currency="UAH",
            ),
            "трыццаць восем грыўнаў і 40 капеек",
        )
        self.assertEqual(
            num2words("1230.56", lang="by", to="currency", currency="USD"),
            "адна тысяча дзвесце трыццаць долараў, пяцьдзясят шэсць цэнтаў",
        )
        self.assertEqual(
            num2words("1231.56", lang="by", to="currency", currency="USD"),
            "адна тысяча дзвесце трыццаць адзін долар, "
            "пяцьдзясят шэсць цэнтаў",
        )
        self.assertEqual(
            num2words("1234.56", lang="by", to="currency", currency="USD"),
            "адна тысяча дзвесце трыццаць чатыры долары, пяцьдзясят шэсць "
            "цэнтаў",
        )
        self.assertEqual(
            num2words(
                10122, lang="by", to="currency", currency="UZS", separator=" і"
            ),
            "сто адзін сум і дваццаць два тыйіны",
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import division

from decimal import ROUND_HALF_UP, Decimal


def parse_currency_parts(value, is_int_with_cents=True):
    if isinstance(value, int):
        if is_int_with_cents:
            # assume cents if value is integer
            negative = value < 0
            value = abs(value)
            integer, cents = divmod(value, 100)
        else:
            negative = value < 0
            integer, cents = abs(value), 0

    else:
        value = Decimal(value)
        value = value.quantize(
            Decimal('.01'),
            rounding=ROUND_HALF_UP
        )
        negative = value < 0
        value = abs(value)
        integer, fraction = divmod(value, 1)
        integer = int(integer)
        cents = int(fraction * 100)

    return integer, cents, negative


def prefix_currency(prefix, base):
    return tuple("%s %s" % (prefix, i) for i in base)



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words
from num2words.lang_NL import Num2Word_NL


class Num2WordsNLTest(TestCase):
    def test_ordinal_less_than_twenty(self):
        self.assertEqual(num2words(7, ordinal=True, lang='nl'), "zevende")
        self.assertEqual(num2words(8, ordinal=True, lang='nl'), "achtste")
        self.assertEqual(num2words(12, ordinal=True, lang='nl'), "twaalfde")
        self.assertEqual(num2words(17, ordinal=True, lang='nl'), "zeventiende")

    def test_ordinal_more_than_twenty(self):
        self.assertEqual(
            num2words(81, ordinal=True, lang='nl'), "eenentachtigste"
        )

    def test_ordinal_at_crucial_number(self):
        self.assertEqual(num2words(0, ordinal=True, lang='nl'), "nulde")
        self.assertEqual(num2words(100, ordinal=True, lang='nl'), "honderdste")
        self.assertEqual(
            num2words(1000, ordinal=True, lang='nl'), "duizendste"
        )
        self.assertEqual(
            num2words(4000, ordinal=True, lang='nl'), "vierduizendste"
        )
        self.assertEqual(
            num2words(2000000, ordinal=True, lang='nl'), "twee miljoenste"
        )
        self.assertEqual(
            num2words(5000000000, ordinal=True, lang='nl'), "vijf miljardste"
        )

    def test_cardinal_at_some_numbers(self):
        self.assertEqual(num2words(82, lang='nl'), u'twee\xebntachtig')
        self.assertEqual(num2words(1013, lang='nl'), "duizenddertien")
        self.assertEqual(num2words(2000000, lang='nl'), "twee miljoen")
        self.assertEqual(num2words(4000000000, lang='nl'), "vier miljard")

    def test_cardinal_for_decimal_number(self):
        self.assertEqual(
            num2words(3.486, lang='nl'), "drie komma vier acht zes"
        )

    def test_ordinal_for_negative_numbers(self):
        self.assertRaises(TypeError, num2words, -12, ordinal=True, lang='nl')

    def test_ordinal_for_floating_numbers(self):
        self.assertRaises(TypeError, num2words, 2.453, ordinal=True, lang='nl')

    def test_to_currency_eur(self):
        self.assertEqual(
            num2words('38.4', lang='nl', to='currency', separator=' en',
                      cents=False, currency='EUR'),
            "achtendertig euro en 40 cent"
        )
        self.assertEqual(
            num2words('0', lang='nl', to='currency', separator=' en',
                      cents=False, currency='EUR'),
            "nul euro en 00 cent"
        )

        self.assertEqual(
            num2words('1.01', lang='nl', to='currency', separator=' en',
                      cents=True, currency='EUR'),
            "één euro en één cent"
        )

        self.assertEqual(
            num2words('4778.00', lang='nl', to='currency', separator=' en',
                      cents=True, currency='EUR'),
            'vierduizendzevenhonderdachtenzeventig euro en nul cent')

    def test_to_currency_usd(self):
        self.assertEqual(
            num2words('38.4', lang='nl', to='currency', separator=' en',
                      cents=False, currency='USD'),
            "achtendertig dollar en 40 cent"
        )
        self.assertEqual(
            num2words('0', lang='nl', to='currency', separator=' en',
                      cents=False, currency='USD'),
            "nul dollar en 00 cent"
        )

        self.assertEqual(
            num2words('1.01', lang='nl', to='currency', separator=' en',
                      cents=True, currency='USD'),
            "één dollar en één cent"
        )

        self.assertEqual(
            num2words('4778.00', lang='nl', to='currency', separator=' en',
                      cents=True, currency='USD'),
            'vierduizendzevenhonderdachtenzeventig dollar en nul cent')

    def test_pluralize(self):
        n = Num2Word_NL()
        # euros always singular
        cr1, cr2 = n.CURRENCY_FORMS['EUR']
        self.assertEqual(n.pluralize(1, cr1), 'euro')
        self.assertEqual(n.pluralize(2, cr1), 'euro')
        self.assertEqual(n.pluralize(1, cr2), 'cent')
        self.assertEqual(n.pluralize(2, cr2), 'cent')

        # @TODO other currency

    def test_to_year(self):
        self.assertEqual(num2words(2018, lang='nl', to='year'),
                         'tweeduizendachttien')
        self.assertEqual(num2words(2100, lang='nl', to='year'),
                         'eenentwintig honderd')



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsLVTest(TestCase):
    def test_to_cardinal(self):
        self.assertEqual(num2words(100, lang='lv'), 'simts')
        self.assertEqual(num2words(101, lang='lv'), 'simtu viens')
        self.assertEqual(num2words(110, lang='lv'), 'simts desmit')
        self.assertEqual(num2words(115, lang='lv'), 'simts piecpadsmit')
        self.assertEqual(num2words(123, lang='lv'), 'simts divdesmit trīs')
        self.assertEqual(num2words(1000, lang='lv'), 'tūkstotis')
        self.assertEqual(num2words(1001, lang='lv'), 'tūkstotis viens')
        self.assertEqual(num2words(2012, lang='lv'),
                         'divi tūkstoši divpadsmit')
        self.assertEqual(
            num2words(1234567890, lang='lv'),
            'miljards divi simti trīsdesmit četri miljoni pieci simti '
            'sešdesmit septiņi tūkstoši astoņi simti deviņdesmit')
        self.assertEqual(
            num2words(215461407892039002157189883901676, lang='lv'),
            'divi simti piecpadsmit nontiljoni četri simti sešdesmit '
            'viens oktiljons četri simti septiņi septiljoni astoņi '
            'simti deviņdesmit divi sikstiljoni trīsdesmit deviņi '
            'kvintiljoni divi kvadriljoni simts piecdesmit septiņi '
            'triljoni simts astoņdesmit deviņi miljardi astoņi simti '
            'astoņdesmit trīs miljoni deviņi simti viens tūkstotis '
            'seši simti septiņdesmit seši')
        self.assertEqual(
            num2words(719094234693663034822824384220291, lang='lv'),
            'septiņi simti deviņpadsmit nontiljoni deviņdesmit četri '
            'oktiljoni divi simti trīsdesmit četri septiljoni seši simti '
            'deviņdesmit trīs sikstiljoni seši simti sešdesmit trīs '
            'kvintiljoni trīsdesmit četri kvadriljoni astoņi simti '
            'divdesmit divi triljoni astoņi simti divdesmit četri '
            'miljardi trīs simti astoņdesmit četri miljoni divi simti '
            'divdesmit tūkstoši divi simti deviņdesmit viens')
        self.assertEqual(
            num2words(-5000, lang='lv'),
            'mīnus pieci tūkstoši',
        )
        self.assertEqual(
            num2words(-5000.22, lang='lv'),
            'mīnus pieci tūkstoši komats divdesmit divi',
        )
        self.assertEqual(
            num2words(10.02, lang='lv'),
            "desmit komats nulle divi"
        )
        self.assertEqual(
            num2words(15.007, lang='lv'),
            "piecpadsmit komats nulle nulle septiņi"
        )

        self.assertEqual(num2words(0, lang='lv'), 'nulle')
        self.assertEqual(num2words(5, lang='lv'), "pieci")
        self.assertEqual(num2words(15, lang='lv'), "piecpadsmit")
        self.assertEqual(num2words(154, lang='lv'), "simts piecdesmit četri")
        self.assertEqual(num2words(101, lang='lv'), "simtu viens")
        self.assertEqual(
            num2words(1135, lang='lv'), "tūkstotis simts trīsdesmit pieci"
        )
        self.assertEqual(
            num2words(418531, lang='lv'),
            "četri simti astoņpadsmit tūkstoši pieci simti trīsdesmit viens"
        )
        self.assertEqual(
            num2words(1000139, lang='lv'),
            "miljons simts trīsdesmit deviņi"
        )

    def test_to_ordinal(self):
        # @TODO: implement to_ordinal
        with self.assertRaises(NotImplementedError):
            num2words(1, lang='lv', to='ordinal')

    def test_to_currency(self):
        self.assertEqual(
            num2words(1.0, lang='lv', to='currency', currency='EUR'),
            "viens eiro, nulle centu"
        )
        self.assertEqual(
            num2words(1.0, lang='lv', to='currency', currency='LVL'),
            "viens lats, nulle santīmu"
        )
        self.assertEqual(
            num2words(1234.56, lang='lv', to='currency', currency='EUR'),
            "tūkstotis divi simti trīsdesmit četri eiro, piecdesmit seši centi"
        )
        self.assertEqual(
            num2words(1234.56, lang='lv', to='currency', currency='LVL'),
            "tūkstotis divi simti trīsdesmit četri lati, "
            "piecdesmit seši santīmi"
        )

        self.assertEqual(
            num2words(10111, lang='lv', to='currency', separator=' un',
                      currency='EUR'),
            "simtu viens eiro un vienpadsmit centi"
        )
        self.assertEqual(
            num2words(10121, lang='lv', to='currency', separator=' un',
                      currency='LVL'),
            "simtu viens lats un divdesmit viens santīms"
        )
        self.assertEqual(
            num2words(-1251985, lang='lv', to='currency', cents=False,
                      currency='EUR'),
            "mīnus divpadsmit tūkstoši pieci simti deviņpadsmit eiro,"
            " 85 centi"
        )
        self.assertEqual(
            num2words('38.4', lang='lv', to='currency', separator=' un',
                      cents=False, currency='EUR'),
            "trīsdesmit astoņi eiro un 40 centi"
        )

        # EUR legal form
        self.assertEqual(
            num2words('38.4', lang='lv', to='currency', separator=' un',
                      cents=False, currency='EUR_LEGAL'),
            "trīsdesmit astoņi euro un 40 centi"
        )

        self.assertEqual(
            num2words('38.4', lang='lv', to='currency', separator=' un',
                      cents=False, currency='USD', adjective=False),
            "trīsdesmit astoņi dolāri un 40 centi"
        )

        self.assertEqual(
            num2words('38.4', lang='lv', to='currency', separator=' un',
                      cents=False, currency='USD', adjective=True),
            "trīsdesmit astoņi ASV dolāri un 40 centi"
        )

    def test_fractions(self):
        self.assertEqual(num2words(5.2, lang='lv'), "pieci komats divi")
        self.assertEqual(
            num2words(561.42, lang='lv'),
            "pieci simti sešdesmit viens komats četrdesmit divi"
        )



# -*- coding, utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA


from unittest import TestCase

from num2words import num2words


class Num2WordAZTest(TestCase):
    lang = 'az'

    CARDINAL_TEST_CASES = (
        (0, 'sıfır',),
        (1, 'bir',),
        (2, 'iki',),
        (3, 'üç',),
        (4, 'dörd',),
        (5, 'beş',),
        (6, 'altı',),
        (7, 'yeddi',),
        (8, 'səkkiz',),
        (9, 'doqquz',),
        (10, 'on',),
        (11, 'on bir',),
        (20, 'iyirmi',),
        (22, 'iyirmi iki',),
        (30, 'otuz',),
        (33, 'otuz üç',),
        (40, 'qırx',),
        (44, 'qırx dörd',),
        (50, 'əlli',),
        (55, 'əlli beş',),
        (60, 'altmış',),
        (66, 'altmış altı',),
        (70, 'yetmiş',),
        (77, 'yetmiş yeddi',),
        (80, 'səksən',),
        (88, 'səksən səkkiz',),
        (90, 'doxsan',),
        (99, 'doxsan doqquz',),
        (100, 'yüz',),
        (200, 'iki yüz',),
        (678, 'altı yüz yetmiş səkkiz',),
        (999, 'doqquz yüz doxsan doqquz',),
        (1000, 'min',),
        (100_000, 'yüz min',),
        (328_914, 'üç yüz iyirmi səkkiz min doqquz yüz on dörd',),
        (1_000_000, 'bir milyon',),
        (1_000_000_000, 'bir milyard',),
        (10**12, 'bir trilyon',),
        (10**15, 'bir katrilyon',),
        (10**18, 'bir kentilyon',),
        (10**21, 'bir sekstilyon',),
        (10**24, 'bir septilyon',),
        (10**27, 'bir oktilyon',),
        (10**30, 'bir nonilyon',),
        (10**33, 'bir desilyon',),
        (10**36, 'bir undesilyon',),
        (10**39, 'bir dodesilyon',),
        (10**42, 'bir tredesilyon',),
        (10**45, 'bir katordesilyon',),
        (10**48, 'bir kendesilyon',),
        (10**51, 'bir seksdesilyon',),
        (10**54, 'bir septendesilyon',),
        (10**57, 'bir oktodesilyon',),
        (10**60, 'bir novemdesilyon',),
        (10**63, 'bir vigintilyon',),
        (-0, 'sıfır',),
        (-1, 'mənfi bir',),
        (-2, 'mənfi iki',),
        (-3, 'mənfi üç',),
        (-4, 'mənfi dörd',),
        (-5, 'mənfi beş',),
        (-6, 'mənfi altı',),
        (-7, 'mənfi yeddi',),
        (-8, 'mənfi səkkiz',),
        (-9, 'mənfi doqquz',),
        (-10, 'mənfi on',),
        (-11, 'mənfi on bir',),
        (-20, 'mənfi iyirmi',),
        (-22, 'mənfi iyirmi iki',),
        (-30, 'mənfi otuz',),
        (-33, 'mənfi otuz üç',),
        (-40, 'mənfi qırx',),
        (-44, 'mənfi qırx dörd',),
        (-50, 'mənfi əlli',),
        (-55, 'mənfi əlli beş',),
        (-60, 'mənfi altmış',),
        (-66, 'mənfi altmış altı',),
        (-70, 'mənfi yetmiş',),
        (-77, 'mənfi yetmiş yeddi',),
        (-80, 'mənfi səksən',),
        (-88, 'mənfi səksən səkkiz',),
        (-90, 'mənfi doxsan',),
        (-99, 'mənfi doxsan doqquz',),
        (-100, 'mənfi yüz',),
        (-200, 'mənfi iki yüz',),
        (-678, 'mənfi altı yüz yetmiş səkkiz',),
        (-999, 'mənfi doqquz yüz doxsan doqquz',),
        (-1000, 'mənfi min',),
        (-100_000, 'mənfi yüz min',),
        (-328_914, 'mənfi üç yüz iyirmi səkkiz min doqquz yüz on dörd',),
        (-1_000_000, 'mənfi bir milyon',),
        (-1_000_000_000, 'mənfi bir milyard',),
        (-10**12, 'mənfi bir trilyon',),
        (-10**15, 'mənfi bir katrilyon',),
        (-10**18, 'mənfi bir kentilyon',),
        (-10**21, 'mənfi bir sekstilyon',),
        (-10**24, 'mənfi bir septilyon',),
        (-10**27, 'mənfi bir oktilyon',),
        (-10**30, 'mənfi bir nonilyon',),
        (-10**33, 'mənfi bir desilyon',),
        (-10**36, 'mənfi bir undesilyon',),
        (-10**39, 'mənfi bir dodesilyon',),
        (-10**42, 'mənfi bir tredesilyon',),
        (-10**45, 'mənfi bir katordesilyon',),
        (-10**48, 'mənfi bir kendesilyon',),
        (-10**51, 'mənfi bir seksdesilyon',),
        (-10**54, 'mənfi bir septendesilyon',),
        (-10**57, 'mənfi bir oktodesilyon',),
        (-10**60, 'mənfi bir novemdesilyon',),
        (-10**63, 'mənfi bir vigintilyon'),
    )

    CARDINAL_FRACTION_TEST_CASES = (
        (0.2, 'sıfır nöqtə iki',),
        (0.02, 'sıfır nöqtə sıfır iki',),
        (0.23, 'sıfır nöqtə iyirmi üç',),
        (0.0023, 'sıfır nöqtə sıfır sıfır iyirmi üç',),
        (1.43, 'bir nöqtə qırx üç',),
        (-0.2, 'mənfi sıfır nöqtə iki',),
        (-0.02, 'mənfi sıfır nöqtə sıfır iki',),
        (-0.23, 'mənfi sıfır nöqtə iyirmi üç',),
        (-0.0023, 'mənfi sıfır nöqtə sıfır sıfır iyirmi üç',),
        (-1.43, 'mənfi bir nöqtə qırx üç',),
    )

    ORDINAL_TEST_CASES = (
        (0, 'sıfırıncı',),
        (1, 'birinci',),
        (2, 'ikinci',),
        (3, 'üçüncü',),
        (4, 'dördüncü',),
        (5, 'beşinci',),
        (6, 'altıncı',),
        (7, 'yeddinci',),
        (8, 'səkkizinci',),
        (9, 'doqquzuncu',),
        (10, 'onuncu',),
        (11, 'on birinci',),
        (20, 'iyirminci',),
        (22, 'iyirmi ikinci',),
        (30, 'otuzuncu',),
        (33, 'otuz üçüncü',),
        (40, 'qırxıncı',),
        (44, 'qırx dördüncü',),
        (50, 'əllinci',),
        (55, 'əlli beşinci',),
        (60, 'altmışıncı',),
        (66, 'altmış altıncı',),
        (70, 'yetmişinci',),
        (77, 'yetmiş yeddinci',),
        (80, 'səksəninci',),
        (88, 'səksən səkkizinci',),
        (90, 'doxsanıncı',),
        (99, 'doxsan doqquzuncu',),
        (100, 'yüzüncü',),
        (1000, 'mininci',),
        (328_914, 'üç yüz iyirmi səkkiz min doqquz yüz on dördüncü',),
        (1_000_000, 'bir milyonuncu'),
    )

    ORDINAL_NUM_TEST_CASES = (
        (0, '0-cı',),
        (1, '1-ci',),
        (2, '2-ci',),
        (3, '3-cü',),
        (4, '4-cü',),
        (5, '5-ci',),
        (6, '6-cı',),
        (7, '7-ci',),
        (8, '8-ci',),
        (9, '9-cu',),
        (10, '10-cu',),
        (11, '11-ci',),
        (20, '20-ci',),
        (22, '22-ci',),
        (30, '30-cu',),
        (33, '33-cü',),
        (40, '40-cı',),
        (44, '44-cü',),
        (50, '50-ci',),
        (55, '55-ci',),
        (60, '60-cı',),
        (66, '66-cı',),
        (70, '70-ci',),
        (77, '77-ci',),
        (80, '80-ci',),
        (88, '88-ci',),
        (90, '90-cı',),
        (99, '99-cu',),
        (100, '100-cü',),
        (1000, '1000-ci',),
        (328_914, '328914-cü',),
        (1_000_000, '1000000-cu'),
    )

    YEAR_TEST_CASES = (
        (167, 'yüz altmış yeddi'),
        (1552, 'min beş yüz əlli iki'),
        (1881, 'min səkkiz yüz səksən bir'),
        (2022, 'iki min iyirmi iki'),
        (-1, 'e.ə. bir'),
        (-500, 'e.ə. beş yüz'),
        (-5000, 'e.ə. beş min'),
    )

    CURRENCY_TEST_CASES = (
        (0.0, 'sıfır manat, sıfır qəpik'),
        (0.01, 'sıfır manat, bir qəpik'),
        (0.1, 'sıfır manat, on qəpik'),
        (1.5, 'bir manat, əlli qəpik'),
        (1.94, 'bir manat, doxsan dörd qəpik'),
        (17.82, 'on yeddi manat, səksən iki qəpik'),
    )

    def test_cardinal(self):
        """Test cardinal conversion for integer numbers."""

        for number, expected in self.CARDINAL_TEST_CASES:
            actual = num2words(number, lang=self.lang, to='cardinal')

            self.assertEqual(actual, expected)

    def test_cardinal_fracion(self):
        """Test cardinal conversion for numbers with fraction."""

        for number, expected in self.CARDINAL_FRACTION_TEST_CASES:
            actual = num2words(number, lang=self.lang, to='cardinal')

            self.assertEqual(actual, expected)

    def test_ordinal(self):
        """Test ordinal conversion."""

        for number, expected in self.ORDINAL_TEST_CASES:
            actual = num2words(number, lang=self.lang, to='ordinal')

            self.assertEqual(actual, expected)

    def test_ordinal_num(self):
        """Test 'ordinal_num' conversion."""

        for number, expected in self.ORDINAL_NUM_TEST_CASES:
            actual = num2words(number, lang=self.lang, to='ordinal_num')

            self.assertEqual(actual, expected)

    def test_year(self):
        """Test year conversion."""

        for number, expected in self.YEAR_TEST_CASES:
            actual = num2words(number, lang=self.lang, to='year')

            self.assertEqual(actual, expected)

    def test_currency(self):
        """Test currency conversion."""

        for number, expected in self.CURRENCY_TEST_CASES:
            actual = num2words(
                number, lang=self.lang, currency='AZN', to='currency')

            self.assertEqual(actual, expected)



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsCZTest(TestCase):
    def test_cardinal(self):
        self.assertEqual(num2words(100, lang='cz'), "sto")
        self.assertEqual(num2words(101, lang='cz'), "sto jedna")
        self.assertEqual(num2words(110, lang='cz'), "sto deset")
        self.assertEqual(num2words(115, lang='cz'), "sto patnáct")
        self.assertEqual(num2words(123, lang='cz'), "sto dvacet tři")
        self.assertEqual(num2words(1000, lang='cz'), "tisíc")
        self.assertEqual(num2words(1001, lang='cz'), "tisíc jedna")
        self.assertEqual(num2words(2012, lang='cz'), "dva tisíce dvanáct")
        self.assertEqual(
            num2words(10.02, lang='cz'),
            "deset celá nula dva"
        )
        self.assertEqual(
            num2words(15.007, lang='cz'),
            "patnáct celá nula nula sedm"
        )
        self.assertEqual(
            num2words(12519.85, lang='cz'),
            "dvanáct tisíc pětset devatenáct celá osmdesát pět"
        )
        self.assertEqual(
            num2words(123.50, lang='cz'),
            "sto dvacet tři celá pět"
        )
        self.assertEqual(
            num2words(1234567890, lang='cz'),
            "miliarda dvěstě třicet čtyři miliony pětset šedesát "
            "sedm tisíc osmset devadesát"
        )
        self.assertEqual(
            num2words(215461407892039002157189883901676, lang='cz'),
            "dvěstě patnáct quintillionů čtyřista šedesát jedna kvadriliard "
            "čtyřista sedm kvadrilionů osmset devadesát dva triliardy třicet "
            "devět trilionů dva biliardy sto padesát sedm bilionů sto "
            "osmdesát devět miliard osmset osmdesát tři miliony "
            "devětset jedna tisíc šestset sedmdesát šest"
        )
        self.assertEqual(
            num2words(719094234693663034822824384220291, lang='cz'),
            "sedmset devatenáct quintillionů devadesát "
            "čtyři kvadriliardy dvěstě třicet čtyři "
            "kvadriliony šestset devadesát tři triliardy "
            "šestset šedesát tři triliony třicet čtyři biliardy osmset "
            "dvacet dva biliony osmset dvacet čtyři "
            "miliardy třista osmdesát čtyři miliony dvěstě dvacet "
            "tisíc dvěstě devadesát jedna"
        )

    def test_to_ordinal(self):
        # @TODO: implement to_ordinal
        with self.assertRaises(NotImplementedError):
            num2words(1, lang='cz', to='ordinal')

    def test_currency(self):
        self.assertEqual(
            num2words(10.0, lang='cz', to='currency', currency='EUR'),
            "deset euro, nula centů")
        self.assertEqual(
            num2words(1.0, lang='cz', to='currency', currency='CZK'),
            "jedna koruna, nula haléřů")
        self.assertEqual(
            num2words(1234.56, lang='cz', to='currency', currency='EUR'),
            "tisíc dvěstě třicet čtyři euro, padesát šest centů")
        self.assertEqual(
            num2words(1234.56, lang='cz', to='currency', currency='CZK'),
            "tisíc dvěstě třicet čtyři koruny, padesát šest haléřů")
        self.assertEqual(
            num2words(101.11, lang='cz', to='currency', currency='EUR',
                      separator=' a'),
            "sto jedna euro a jedenáct centů")
        self.assertEqual(
            num2words(101.21, lang='cz', to='currency', currency='CZK',
                      separator=' a'),
            "sto jedna korun a dvacet jedna haléřů"
        )
        self.assertEqual(
            num2words(-12519.85, lang='cz', to='currency', cents=False),
            "mínus dvanáct tisíc pětset devatenáct euro, 85 centů"
        )
        self.assertEqual(
            num2words(123.50, lang='cz', to='currency', currency='CZK',
                      separator=' a'),
            "sto dvacet tři koruny a padesát haléřů"
        )
        self.assertEqual(
            num2words(19.50, lang='cz', to='currency', cents=False),
            "devatenáct euro, 50 centů"
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA


from unittest import TestCase

from num2words import num2words


class Num2WordsKNTest(TestCase):
    def test_numbers(self):
        self.assertEqual(num2words(42, lang="kn"), u"ನಲವತ್ತ್ ಎರಡು")
        self.assertEqual(num2words(893, lang="kn"), u"ಎಂಟು ನೂರ ತೊಂಬತ್ತ ಮೂರು")
        self.assertEqual(
            num2words(1729, lang="kn"), u"ಒಂದು ಸಾವಿರ ಏಳು ನೂರ ಇಪ್ಪತ್ತ್ಒಂಬತ್ತು"
        )
        self.assertEqual(num2words(123, lang="kn"), u"ಒಂದು ನೂರ ಇಪ್ಪತ್ತ್ ಮೂರು")
        self.assertEqual(num2words(32211, lang="kn"),
                         u"ಮೂವತ್ತ್ಎರಡು ಸಾವಿರ ಎರಡು ನೂರ ಹನ್ನೊಂದು")

    def test_cardinal_for_float_number(self):
        self.assertEqual(num2words(3.14, lang="kn"), u"ಮೂರು ಬಿಂದು ಒಂದು ನಾಲ್ಕು")
        self.assertEqual(num2words(1.61803, lang="kn"),
                         u"ಒಂದು ಬಿಂದು ಆರು ಒಂದು ಎಂಟು ಸೊನ್ನೆ ಮೂರು")

    def test_ordinal(self):
        self.assertEqual(
            num2words(1, lang='kn', to='ordinal'),
            u'ಒಂದನೇ'
        )
        self.assertEqual(
            num2words(22, lang='kn', to='ordinal'),
            u'ಇಪ್ಪತ್ತ್ ಎರಡನೇ'
        )
        self.assertEqual(
            num2words(12, lang='kn', to='ordinal'),
            u'ಹನ್ನೆರಡನೇ'
        )
        self.assertEqual(
            num2words(130, lang='kn', to='ordinal'),
            u'ಒಂದು ನೂರ ಮೂವತ್ತನೇ'
        )
        self.assertEqual(
            num2words(1003, lang='kn', to='ordinal'),
            u'ಒಂದು ಸಾವಿರದ ಮೂರನೇ'
        )
        self.assertEqual(num2words(2, lang="kn", ordinal=True), u"ಎರಡನೇ")
        self.assertEqual(num2words(5, lang="kn", ordinal=True), u"ಐದನೇ")
        self.assertEqual(num2words(16, lang="kn", ordinal=True), u"ಹದಿನಾರನೇ")
        self.assertEqual(num2words(113, lang="kn", ordinal=True),
                         u"ಒಂದು ನೂರ ಹದಿಮೂರನೇ")

    def test_ordinal_num(self):
        self.assertEqual(num2words(2, lang="kn", to='ordinal_num'), u"2ಎರಡನೇ")
        self.assertEqual(num2words(5, lang="kn", to='ordinal_num'), u"5ಐದನೇ")
        self.assertEqual(num2words(16, lang="kn", to='ordinal_num'),
                         u"16ಹದಿನಾರನೇ")
        self.assertEqual(num2words(113, lang="kn", to='ordinal_num'),
                         u"113ಒಂದು ನೂರ ಹದಿಮೂರನೇ")



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import print_function, unicode_literals

import re

from .lang_EU import Num2Word_EU


class Num2Word_DE(Num2Word_EU):
    CURRENCY_FORMS = {
        'EUR': (('Euro', 'Euro'), ('Cent', 'Cent')),
        'GBP': (('Pfund', 'Pfund'), ('Penny', 'Pence')),
        'USD': (('Dollar', 'Dollar'), ('Cent', 'Cent')),
        'CNY': (('Yuan', 'Yuan'), ('Jiao', 'Fen')),
        'DEM': (('Mark', 'Mark'), ('Pfennig', 'Pfennig')),
    }

    GIGA_SUFFIX = "illiarde"
    MEGA_SUFFIX = "illion"

    def setup(self):
        self.negword = "minus "
        self.pointword = "Komma"
        # "Cannot treat float %s as ordinal."
        self.errmsg_floatord = (
            "Die Gleitkommazahl %s kann nicht in eine Ordnungszahl " +
            "konvertiert werden."
            )
        # "type(((type(%s)) ) not in [long, int, float]"
        self.errmsg_nonnum = (
            "Nur Zahlen (type(%s)) können in Wörter konvertiert werden."
            )
        # "Cannot treat negative num %s as ordinal."
        self.errmsg_negord = (
            "Die negative Zahl %s kann nicht in eine Ordnungszahl " +
            "konvertiert werden."
            )
        # "abs(%s) must be less than %s."
        self.errmsg_toobig = "Die Zahl %s muss kleiner als %s sein."
        self.exclude_title = []

        lows = ["Non", "Okt", "Sept", "Sext", "Quint", "Quadr", "Tr", "B", "M"]
        units = ["", "un", "duo", "tre", "quattuor", "quin", "sex", "sept",
                 "okto", "novem"]
        tens = ["dez", "vigint", "trigint", "quadragint", "quinquagint",
                "sexagint", "septuagint", "oktogint", "nonagint"]
        self.high_numwords = (
            ["zent"] + self.gen_high_numwords(units, tens, lows)
        )
        self.mid_numwords = [(1000, "tausend"), (100, "hundert"),
                             (90, "neunzig"), (80, "achtzig"), (70, "siebzig"),
                             (60, "sechzig"), (50, "f\xFCnfzig"),
                             (40, "vierzig"), (30, "drei\xDFig")]
        self.low_numwords = ["zwanzig", "neunzehn", "achtzehn", "siebzehn",
                             "sechzehn", "f\xFCnfzehn", "vierzehn", "dreizehn",
                             "zw\xF6lf", "elf", "zehn", "neun", "acht",
                             "sieben", "sechs", "f\xFCnf", "vier", "drei",
                             "zwei", "eins", "null"]
        self.ords = {"eins": "ers",
                     "drei": "drit",
                     "acht": "ach",
                     "sieben": "sieb",
                     "ig": "igs",
                     "ert": "erts",
                     "end": "ends",
                     "ion": "ions",
                     "nen": "ns",
                     "rde": "rds",
                     "rden": "rds"}

    def merge(self, curr, next):
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1:
            if nnum == 100 or nnum == 1000:
                return ("ein" + ntext, nnum)
            elif nnum < 10 ** 6:
                return next
            ctext = "eine"

        if nnum > cnum:
            if nnum >= 10 ** 6:
                if cnum > 1:
                    if ntext.endswith("e"):
                        ntext += "n"
                    else:
                        ntext += "en"
                ctext += " "
            val = cnum * nnum
        else:
            if nnum < 10 < cnum < 100:
                if nnum == 1:
                    ntext = "ein"
                ntext, ctext = ctext, ntext + "und"
            elif cnum >= 10 ** 6:
                ctext += " "
            val = cnum + nnum

        word = ctext + ntext
        return (word, val)

    def to_ordinal(self, value):
        self.verify_ordinal(value)
        outword = self.to_cardinal(value).lower()
        for key in self.ords:
            if outword.endswith(key):
                outword = outword[:len(outword) - len(key)] + self.ords[key]
                break

        res = outword + "te"

        # Exception: "hundertste" is usually preferred over "einhundertste"
        if res == "eintausendste" or res == "einhundertste":
            res = res.replace("ein", "", 1)
        # ... similarly for "millionste" etc.
        res = re.sub(r'eine ([a-z]+(illion|illiard)ste)$',
                     lambda m: m.group(1), res)
        # Ordinals involving "Million" etc. are written without a space.
        # see https://de.wikipedia.org/wiki/Million#Sprachliches
        res = re.sub(r' ([a-z]+(illion|illiard)ste)$',
                     lambda m: m.group(1), res)

        return res

    def to_ordinal_num(self, value):
        self.verify_ordinal(value)
        return str(value) + "."

    def to_currency(self, val, currency='EUR', cents=True, separator=' und',
                    adjective=False):
        result = super(Num2Word_DE, self).to_currency(
            val, currency=currency, cents=cents, separator=separator,
            adjective=adjective)
        # Handle exception, in german is "ein Euro" and not "eins Euro"
        return result.replace("eins ", "ein ")

    def to_year(self, val, longval=True):
        if not (val // 100) % 10:
            return self.to_cardinal(val)
        return self.to_splitnum(val, hightxt="hundert", longval=longval)\
            .replace(' ', '')



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from unittest import TestCase

from num2words import num2words


class Num2WordsARTest(TestCase):

    def test_default_currency(self):
        self.assertEqual(num2words(1, to='currency', lang='ar'), 'واحد ريال')
        self.assertEqual(num2words(2, to='currency', lang='ar'),
                         'اثنان ريالان')
        self.assertEqual(num2words(10, to='currency', lang='ar'),
                         'عشرة ريالات')
        self.assertEqual(num2words(100, to='currency', lang='ar'), 'مائة ريال')
        self.assertEqual(num2words(652.12, to='currency', lang='ar'),
                         'ستمائة و اثنان و خمسون ريالاً و اثنتا عشرة هللة')
        self.assertEqual(num2words(324, to='currency', lang='ar'),
                         'ثلاثمائة و أربعة و عشرون ريالاً')
        self.assertEqual(num2words(2000, to='currency', lang='ar'),
                         'ألفا ريال')
        self.assertEqual(num2words(541, to='currency', lang='ar'),
                         'خمسمائة و واحد و أربعون ريالاً')
        self.assertEqual(num2words(10000, to='currency', lang='ar'),
                         'عشرة آلاف ريال')
        self.assertEqual(num2words(20000.12, to='currency', lang='ar'),
                         'عشرون ألف ريال و اثنتا عشرة هللة')
        self.assertEqual(num2words(1000000, to='currency', lang='ar'),
                         'مليون ريال')
        val = 'تسعمائة و ثلاثة و عشرون ألفاً و أربعمائة و أحد عشر ريالاً'
        self.assertEqual(num2words(923411, to='currency', lang='ar'), val)
        self.assertEqual(num2words(63411, to='currency', lang='ar'),
                         'ثلاثة و ستون ألفاً و أربعمائة و أحد عشر ريالاً')
        self.assertEqual(num2words(1000000.99, to='currency', lang='ar'),
                         'مليون ريال و تسع و تسعون هللة')

    def test_currency_parm(self):
        self.assertEqual(
            num2words(1, to='currency', lang='ar', currency="KWD"),
            'واحد دينار')
        self.assertEqual(
            num2words(10, to='currency', lang='ar', currency="EGP"),
            'عشرة جنيهات')
        self.assertEqual(
            num2words(20000.12, to='currency', lang='ar', currency="EGP"),
            'عشرون ألف جنيه و اثنتا عشرة قرش')
        self.assertEqual(
            num2words(923411, to='currency', lang='ar', currency="SR"),
            'تسعمائة و ثلاثة و عشرون ألفاً و أربعمائة و أحد عشر ريالاً')
        self.assertEqual(
            num2words(1000000.99, to='currency', lang='ar', currency="KWD"),
            'مليون دينار و تسع و تسعون فلس')

    def test_ordinal(self):

        self.assertEqual(num2words(1, to='ordinal', lang='ar'), 'اول')
        self.assertEqual(num2words(2, to='ordinal', lang='ar'), 'ثاني')
        self.assertEqual(num2words(3, to='ordinal', lang='ar'), 'ثالث')
        self.assertEqual(num2words(4, to='ordinal', lang='ar'), 'رابع')
        self.assertEqual(num2words(5, to='ordinal', lang='ar'), 'خامس')
        self.assertEqual(num2words(6, to='ordinal', lang='ar'), 'سادس')
        self.assertEqual(num2words(9, to='ordinal', lang='ar'), 'تاسع')
        self.assertEqual(num2words(20, to='ordinal', lang='ar'), 'عشرون')
        self.assertEqual(num2words(94, to='ordinal', lang='ar'),
                         'أربع و تسعون')
        self.assertEqual(num2words(102, to='ordinal', lang='ar'),
                         'مائة و اثنان')
        self.assertEqual(
            num2words(923411, to='ordinal_num', lang='ar'),
            'تسعمائة و ثلاثة و عشرون ألفاً و أربعمائة و أحد عشر')

        # See https://github.com/savoirfairelinux/num2words/issues/403
        self.assertEqual(num2words(23, lang="ar"), 'ثلاثة و عشرون')
        self.assertEqual(num2words(23, to='ordinal',
                         lang="ar"), 'ثلاث و عشرون')
        self.assertEqual(num2words(23, lang="ar"), 'ثلاثة و عشرون')

    def test_cardinal(self):
        self.assertEqual(num2words(0, to='cardinal', lang='ar'), 'صفر')
        self.assertEqual(num2words(12, to='cardinal', lang='ar'), 'اثنا عشر')
        self.assertEqual(num2words(12.3, to='cardinal', lang='ar'),
                         'اثنا عشر  , ثلاثون')
        self.assertEqual(num2words(12.01, to='cardinal', lang='ar'),
                         'اثنا عشر  , إحدى')
        self.assertEqual(num2words(12.02, to='cardinal', lang='ar'),
                         'اثنا عشر  , اثنتان')
        self.assertEqual(num2words(12.03, to='cardinal', lang='ar'),
                         'اثنا عشر  , ثلاث')
        self.assertEqual(num2words(12.34, to='cardinal', lang='ar'),
                         'اثنا عشر  , أربع و ثلاثون')
        # Not implemented
        self.assertEqual(num2words(12.345, to='cardinal', lang='ar'),
                         num2words(12.34, to='cardinal', lang='ar'))
        self.assertEqual(num2words(-8324, to='cardinal', lang='ar'),
                         'سالب ثمانية آلاف و ثلاثمائة و أربعة و عشرون')

        self.assertEqual(num2words(200, to='cardinal', lang='ar'),
                         'مئتا')
        self.assertEqual(num2words(700, to='cardinal', lang='ar'),
                         'سبعمائة')
        self.assertEqual(num2words(101010, to='cardinal', lang='ar'),
                         'مائة و ألف ألف و عشرة')

        self.assertEqual(
            num2words(3431.12, to='cardinal', lang='ar'),
            'ثلاثة آلاف و أربعمائة و واحد و ثلاثون  , اثنتا عشرة')
        self.assertEqual(num2words(431, to='cardinal', lang='ar'),
                         'أربعمائة و واحد و ثلاثون')
        self.assertEqual(num2words(94231, to='cardinal', lang='ar'),
                         'أربعة و تسعون ألفاً و مئتان و واحد و ثلاثون')
        self.assertEqual(num2words(1431, to='cardinal', lang='ar'),
                         'ألف و أربعمائة و واحد و ثلاثون')
        self.assertEqual(num2words(740, to='cardinal', lang='ar'),
                         'سبعمائة و أربعون')
        self.assertEqual(num2words(741, to='cardinal', lang='ar'),
                         # 'سبعة مائة و واحد و أربعون'
                         'سبعمائة و واحد و أربعون'
                         )
        self.assertEqual(num2words(262, to='cardinal', lang='ar'),
                         'مئتان و اثنان و ستون'
                         )
        self.assertEqual(num2words(798, to='cardinal', lang='ar'),
                         'سبعمائة و ثمانية و تسعون'
                         )
        self.assertEqual(num2words(710, to='cardinal', lang='ar'),
                         'سبعمائة و عشرة')
        self.assertEqual(num2words(711, to='cardinal', lang='ar'),
                         # 'سبعة مائة و إحدى عشر'
                         'سبعمائة و أحد عشر'
                         )
        self.assertEqual(num2words(700, to='cardinal', lang='ar'),
                         'سبعمائة')
        self.assertEqual(num2words(701, to='cardinal', lang='ar'),
                         'سبعمائة و واحد')

        self.assertEqual(
            num2words(1258888, to='cardinal', lang='ar'),
            'مليون و مئتان و ثمانية و خمسون ألفاً و ثمانمائة و ثمانية و ثمانون'
        )

        self.assertEqual(num2words(1100, to='cardinal', lang='ar'),
                         'ألف و مائة')

        self.assertEqual(num2words(1000000521, to='cardinal', lang='ar'),
                         'مليار و خمسمائة و واحد و عشرون')

    def test_prefix_and_suffix(self):
        self.assertEqual(num2words(645, to='currency',
                                   lang='ar', prefix="فقط", suffix="لاغير"),
                         'فقط ستمائة و خمسة و أربعون ريالاً لاغير')

    def test_year(self):
        self.assertEqual(num2words(2000, to='year', lang='ar'), 'ألفا')

    def test_max_numbers(self):

        for number in 10**51, 10**51 + 2:

            with self.assertRaises(OverflowError) as context:
                num2words(number, lang='ar')

            self.assertTrue('must be less' in str(context.exception))

    def test_big_numbers(self):
        self.assertEqual(
            num2words(1000000045000000000000003000000002000000300,
                      to='cardinal', lang='ar'),
            'تريديسيليون و خمسة و أربعون ديسيليوناً\
 و ثلاثة كوينتليونات و ملياران و ثلاثمائة'
        )
        self.assertEqual(
            num2words(-1000000000000000000000003000000002000000302,
                      to='cardinal', lang='ar'),
            'سالب تريديسيليون و ثلاثة كوينتليونات \
و ملياران و ثلاثمائة و اثنان'
        )
        self.assertEqual(
            num2words(9999999999999999999999999999999999999999999999992,
                      to='cardinal', lang='ar'),
            'تسعة كوينتينيليونات و تسعمائة و\
 تسعة و تسعون كوادريسيليوناً و تسعمائة و تسعة\
 و تسعون تريديسيليوناً و تسعمائة و تسعة و تسعون دوديسيليوناً و تسعمائة\
 و تسعة و تسعون أندسيليوناً و تسعمائة و تسعة و تسعون ديسيليوناً\
 و تسعمائة و تسعة و تسعون نونيليوناً و تسعمائة و تسعة و تسعون\
 أوكتيليوناً و تسعمائة و تسعة و تسعون سبتيليوناً و تسعمائة و تسعة\
 و تسعون سكستيليوناً و تسعمائة و تسعة و تسعون كوينتليوناً و تسعمائة و\
 تسعة و تسعون كوادريليوناً و تسعمائة و تسعة و تسعون تريليوناً\
 و تسعمائة و تسعة و تسعون ملياراً و تسعمائة و تسعة و تسعون مليوناً\
 و تسعمائة و تسعة و تسعون ألفاً و تسعمائة و اثنان و تسعون'
        )
